#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: templates_template_id_versions_commit
short_description: Resource module for Templates Template Id Versions Commit
description:
  - Manage operation create of the resource Templates Template Id Versions Commit.
  - Transitions the current draft of a template to a new committed version with a higher version number.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  commitNote:
    description: A message to leave as a note with the commit of a template. The maximum length allowed is 255 characters.
    type: str
  templateId:
    description: TemplateId path parameter. The id of the template to commit a new version for, retrieveable from `GET /dna/intent/api/v1/templates`.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Configuration Templates CommitTemplateForANewVersion
    description: Complete reference of the CommitTemplateForANewVersion API.
    link: https://developer.cisco.com/docs/dna-center/#!commit-template-for-a-new-version
notes:
  - SDK Method used are
    configuration_templates.ConfigurationTemplates.commit_template_for_a_new_version,
  - Paths used are
    post /dna/intent/api/v1/templates/{templateId}/versions/commit,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.templates_template_id_versions_commit:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    commitNote: string
    templateId: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "count": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_info
short_description: Information module for Network Devices
description:
  - Get all Network Devices.
  - Get Network Devices by id.
  - Gets the Network Device details based on the provided query parameters.
  - When there is no start and end time specified returns the latest device details.
  - For detailed information about the usage of the API, please refer to the Open API specification document
    https //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/
    CE_Cat_Center_Org-AssuranceNetworkDevices-2.0.1-resolved.yaml.
  - Returns the device data for the given device Uuid in the specified start and end time range.
  - When there is no start and end time specified returns the latest available data for the given Id.
  - For detailed information about the usage of the API, please refer to the Open API specification document
    https //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/
    CE_Cat_Center_Org-AssuranceNetworkDevices-2.0.1-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description:
      - >
        StartTime query parameter. Start time from which API queries the data set related to the resource. It
        must be specified in UNIX epochtime in milliseconds. Value is inclusive. If `startTime` is not provided,
        API will default to current time.
    type: float
  endTime:
    description:
      - >
        EndTime query parameter. End time to which API queries the data set related to the resource. It must be
        specified in UNIX epochtime in milliseconds. Value is inclusive.
    type: float
  limit:
    description:
      - Limit query parameter. Maximum number of records to return.
    type: int
  offset:
    description:
      - >
        Offset query parameter. Specifies the starting point within all records returned by the API. It's one
        based offset. The starting value is 1.
    type: int
  sortBy:
    description:
      - SortBy query parameter. A field within the response to sort by.
    type: str
  order:
    description:
      - Order query parameter. The sort order of the field ascending or descending.
    type: str
  siteHierarchy:
    description:
      - >
        SiteHierarchy query parameter. The full hierarchical breakdown of the site tree starting from Global
        site name and ending with the specific site name. The Root site is named "Global" (Ex.
        `Global/AreaName/BuildingName/FloorName`) This field supports wildcard asterisk (*) character search
        support. E.g. */San*, */San, /San* Examples `?siteHierarchy=Global/AreaName/BuildingName/FloorName`
        (single siteHierarchy requested) `?siteHierarchy=Global/AreaName/BuildingName/FloorName&siteHierarchy=Gl
        obal/AreaName2/BuildingName2/FloorName2` (multiple siteHierarchies requested).
    type: str
  siteHierarchyId:
    description:
      - >
        SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
        Global site UUID and ending with the specific site UUID. (Ex.
        `globalUuid/areaUuid/buildingUuid/floorUuid`) This field supports wildcard asterisk (*) character search
        support. E.g. `*uuid*, *uuid, uuid* Examples
        `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteH
        ierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2
        /floorUuid2` (multiple siteHierarchyIds requested).
    type: str
  siteId:
    description:
      - >
        SiteId query parameter. The UUID of the site. (Ex. `flooruuid`) This field supports wildcard asterisk
        (*) character search support. E.g.*flooruuid*, *flooruuid, flooruuid* Examples `?siteId=id1` (single id
        requested) `?siteId=id1&siteId=id2&siteId=id3` (multiple ids requested).
    type: str
  id:
    description:
      - >
        Id query parameter. The list of entity Uuids. (Ex."6bef213c-19ca-4170-8375-b694e251101c") Examples
        id=6bef213c-19ca-4170-8375-b694e251101c (single entity uuid requested) id=6bef213c-19ca-4170-8375-
        b694e251101c&id=32219612-819e-4b5e-a96b-cf22aca13dd9&id=2541e9a7-b80d-4955-8aa2-79b233318ba0 (multiple
        entity uuid with '&' separator).
    type: str
  managementIpAddress:
    description:
      - >
        ManagementIpAddress query parameter. The list of entity management IP Address. It can be either Ipv4 or
        Ipv6 address or combination of both(Ex. "121.1.1.10") This field supports wildcard (`*`) character-based
        search. Ex `*1.1*` or `1.1*` or `*1.1` Examples managementIpAddresses=121.1.1.10
        managementIpAddresses=121.1.1.10&managementIpAddresses=172.20.1.10&managementIpAddresses=200
        10&=managementIpAddresses172.20.3.4 (multiple entity IP Address with & separator).
    type: str
  macAddress:
    description:
      - >
        MacAddress query parameter. The macAddress of the network device or client This field supports wildcard
        (`*`) character-based search. Ex `*AB AB AB*` or `AB AB AB*` or `*AB AB AB` Examples `macAddress=AB AB
        AB CD CD CD` (single macAddress requested) `macAddress=AB AB AB CD CD DC&macAddress=AB AB AB CD CD FE`
        (multiple macAddress requested).
    type: str
  family:
    description:
      - >
        Family query parameter. The list of network device family names Examples family=Switches and Hubs
        (single network device family name )family=Switches and Hubs&family=Router&family=Wireless Controller
        (multiple Network device family names with & separator). This field is not case sensitive.
    type: str
  type:
    description:
      - >
        Type query parameter. The list of network device type This field supports wildcard (`*`) character-based
        search. Ex `*9407R*` or `*9407R` or `9407R*` Examples type=SwitchesCisco Catalyst 9407R Switch (single
        network device types ) type=Cisco Catalyst 38xx stack-able ethernet switch&type=Cisco 3945 Integrated
        Services Router G2 (multiple Network device types with & separator).
    type: str
  role:
    description:
      - >
        Role query parameter. The list of network device role. Examples role=CORE,
        role=CORE&role=ACCESS&role=ROUTER (multiple Network device roles with & separator). This field is not
        case sensitive.
    type: str
  serialNumber:
    description:
      - >
        SerialNumber query parameter. The list of network device serial numbers. This field supports wildcard
        (`*`) character-based search. Ex `*MS1SV*` or `MS1SV*` or `*MS1SV` Examples serialNumber=9FUFMS1SVAX
        serialNumber=9FUFMS1SVAX&FCW2333Q0BY&FJC240617JX(multiple Network device serial number with &
        separator).
    type: str
  maintenanceMode:
    description:
      - MaintenanceMode query parameter. The device maintenanceMode status true or false.
    type: bool
  softwareVersion:
    description:
      - >
        SoftwareVersion query parameter. The list of network device software version This field supports
        wildcard (`*`) character-based search. Ex `*17.8*` or `*17.8` or `17.8*` Examples
        softwareVersion=2.3.4.0 (single network device software version )
        softwareVersion=17.9.3.23&softwareVersion=17.7.1.2&softwareVersion=*.17.7 (multiple Network device
        software versions with & separator).
    type: str
  healthScore:
    description:
      - >
        HealthScore query parameter. The list of entity health score categories Examples healthScore=good,
        healthScore=good&healthScore=fair (multiple entity healthscore values with & separator). This field is
        not case sensitive.
    type: str
  view:
    description:
      - >
        View query parameter. The List of Network Device model views. Please refer to ```NetworkDeviceView```
        section in the Open API specification document mentioned in the description.
    type: str
  attribute:
    description:
      - >
        Attribute query parameter. The List of Network Device model attributes. Please refer to
        ```NetworkDeviceAttribute``` section in the Open API specification document mentioned in the
        description.
    type: str
  fabricSiteId:
    description:
      - >
        FabricSiteId query parameter. The fabric site Id or list to fabric site Ids to filter the data This
        field supports wildcard asterisk (*) character search support. E.g. *uuid*, *uuid, uuid* Examples
        `?fabricSiteId=fabricSiteUuid) ?fabricSiteId=fabricSiteUuid1&fabricSiteId=fabricSiteUuid2 (multiple
        fabricSiteIds requested).
    type: str
  l2Vn:
    description:
      - >
        L2Vn query parameter. The L2 Virtual Network Id or list to Virtual Network Ids to filter the data This
        field supports wildcard asterisk (*) character search support. E.g. *uuid*, *uuid, uuid* Examples
        `?l2Vn=virtualNetworkId ?l2Vn=virtualNetworkId1&l2Vn=virtualNetworkId2 (multiple virtualNetworkId's
        requested).
    type: str
  l3Vn:
    description:
      - >
        L3Vn query parameter. The L3 Virtual Network Id or list to Virtual Network Ids to filter the data This
        field supports wildcard asterisk (*) character search support. E.g. *uuid*, *uuid, uuid* Examples
        `?l3Vn=virtualNetworkId ?l3Vn=virtualNetworkId1&l3Vn=virtualNetworkId2 (multiple virtualNetworkId's
        requested).
    type: str
  transitNetworkId:
    description:
      - >
        TransitNetworkId query parameter. The Transit Network Id or list to Transit Network Ids to filter the
        data This field supports wildcard asterisk (*) character search support. E.g. *uuid*, *uuid, uuid*
        Examples `?transitNetworkId=transitNetworkId
        ?transitNetworkId=transitNetworkuuid1&transitNetworkId=transitNetworkuuid1 (multiple transitNetworkIds
        requested.
    type: str
  fabricRole:
    description:
      - >
        FabricRole query parameter. The list of fabric device role. Examples fabricRole=BORDER,
        fabricRole=BORDER&fabricRole=EDGE (multiple fabric device roles with & separator) Available values
        BORDER, EDGE, MAP-SERVER, LEAF, SPINE, TRANSIT-CP, EXTENDED-NODE, WLC, UNIFIED-AP.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Devices GetTheDeviceDataForTheGivenDeviceIdUuid
    description: Complete reference of the GetTheDeviceDataForTheGivenDeviceIdUuid API.
    link: https://developer.cisco.com/docs/dna-center/#!get-the-device-data-for-the-given-device-id-uuid
  - name: Cisco DNA Center documentation for Devices GetsTheNetworkDeviceDetailsBasedOnTheProvidedQueryParameters
    description: Complete reference of the GetsTheNetworkDeviceDetailsBasedOnTheProvidedQueryParameters API.
    link: https://developer.cisco.com/docs/dna-center/#!gets-the-network-device-details-based-on-the-provided-query-parameters
notes:
  - SDK Method used are
    devices.Devices.get_the_device_data_for_the_given_device_id_uuid,
    devices.Devices.gets_the_network_device_details_based_on_the_provided_query_parameters,
  - Paths used are
    get /dna/data/api/v1/networkDevices,
    get /dna/data/api/v1/networkDevices/{id},
"""

EXAMPLES = r"""
---
- name: Get all Network Devices
  cisco.dnac.network_devices_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    limit: 0
    offset: 0
    sortBy: string
    order: string
    siteHierarchy: string
    siteHierarchyId: string
    siteId: string
    id: string
    managementIpAddress: string
    macAddress: string
    family: string
    type: string
    role: string
    serialNumber: string
    maintenanceMode: true
    softwareVersion: string
    healthScore: string
    view: string
    attribute: string
    fabricSiteId: string
    l2Vn: string
    l3Vn: string
    transitNetworkId: string
    fabricRole: string
  register: result
- name: Get Network Devices by id
  cisco.dnac.network_devices_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    view: string
    attribute: string
    id: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "name": "string",
        "managementIpAddress": "string",
        "platformId": "string",
        "deviceFamily": "string",
        "serialNumber": "string",
        "macAddress": "string",
        "deviceSeries": "string",
        "softwareVersion": "string",
        "productVendor": "string",
        "deviceRole": "string",
        "deviceType": "string",
        "communicationState": "string",
        "reachabilityHealthStatus": "string",
        "collectionStatus": "string",
        "haStatus": "string",
        "lastBootTime": 0,
        "siteHierarchyId": "string",
        "siteHierarchy": "string",
        "siteId": "string",
        "deviceGroupHierarchyId": "string",
        "tagNames": [
          "string"
        ],
        "stackType": "string",
        "osType": "string",
        "ringStatus": true,
        "maintenanceModeEnabled": true,
        "upTime": 0,
        "ipv4Address": "string",
        "ipv6Address": "string",
        "redundancyMode": "string",
        "featureFlagList": [
          "string"
        ],
        "haLastResetReason": "string",
        "redundancyPeerStateDerived": "string",
        "redundancyPeerState": "string",
        "redundancyStateDerived": "string",
        "redundancyState": "string",
        "wiredClientCount": 0,
        "wirelessClientCount": 0,
        "portCount": 0,
        "physicalPortCount": 0,
        "virtualPortCount": 0,
        "clientCount": 0,
        "apDetails": {
          "connectedWlcName": "string",
          "policyTagName": "string",
          "apOperationalState": "string",
          "powerSaveMode": "string",
          "operationalMode": "string",
          "resetReason": "string",
          "protocol": "string",
          "powerMode": "string",
          "connectedTime": 0,
          "ledFlashEnabled": true,
          "ledFlashSeconds": 0,
          "subMode": "string",
          "homeApEnabled": true,
          "powerType": "string",
          "apType": "string",
          "adminState": "string",
          "icapCapability": "string",
          "regulatoryDomain": "string",
          "ethernetMac": "string",
          "rfTagName": "string",
          "siteTagName": "string",
          "powerSaveModeCapable": "string",
          "powerProfile": "string",
          "flexGroup": "string",
          "powerCalendarProfile": "string",
          "apGroup": "string",
          "radios": [
            {
              "id": "string",
              "band": "string",
              "noise": 0,
              "airQuality": 0,
              "interference": 0,
              "trafficUtil": 0,
              "utilization": 0,
              "clientCount": 0
            }
          ]
        },
        "metricsDetails": {
          "overallHealthScore": 0,
          "cpuUtilization": 0,
          "cpuScore": 0,
          "memoryUtilization": 0,
          "memoryScore": 0,
          "avgTemperature": 0,
          "maxTemperature": 0,
          "discardScore": 0,
          "discardInterfaces": [
            "string"
          ],
          "errorScore": 0,
          "errorInterfaces": [
            "string"
          ],
          "interDeviceLinkScore": 0,
          "interDeviceConnectedDownInterfaces": [
            "string"
          ],
          "linkUtilizationScore": 0,
          "highLinkUtilizationInterfaces": [
            "string"
          ],
          "freeTimerScore": 0,
          "freeTimer": 0,
          "packetPoolScore": 0,
          "packetPool": 0,
          "freeMemoryBufferScore": 0,
          "freeMemoryBuffer": 0,
          "wqePoolScore": 0,
          "wqePool": 0,
          "apCount": 0,
          "noiseScore": 0,
          "utilizationScore": 0,
          "interferenceScore": 0,
          "airQualityScore": 0
        },
        "fabricDetails": {
          "fabricRole": [
            "string"
          ],
          "fabricSiteName": "string",
          "transitFabrics": [
            "string"
          ],
          "l2Vns": [
            "string"
          ],
          "l3Vns": [
            "string"
          ],
          "fabricSiteId": "string",
          "networkProtocol": "string"
        },
        "switchPoeDetails": {
          "portCount": 0,
          "usedPortCount": 0,
          "freePortCount": 0,
          "powerConsumed": 0,
          "poePowerConsumed": 0,
          "systemPowerConsumed": 0,
          "powerBudget": 0,
          "poePowerAllocated": 0,
          "systemPowerAllocated": 0,
          "powerRemaining": 0,
          "poeVersion": "string",
          "chassisCount": 0,
          "moduleCount": 0,
          "moduleDetails": [
            {
              "moduleId": "string",
              "chassisId": "string",
              "modulePortCount": 0,
              "moduleUsedPortCount": 0,
              "moduleFreePortCount": 0,
              "modulePowerConsumed": 0,
              "modulePoePowerConsumed": 0,
              "moduleSystemPowerConsumed": 0,
              "modulePowerBudget": 0,
              "modulePoePowerAllocated": 0,
              "moduleSystemPowerAllocated": 0,
              "modulePowerRemaining": 0,
              "interfacePowerMax": 0
            }
          ]
        },
        "fabricMetricsDetails": {
          "overallFabricScore": 0,
          "fabricTransitScore": 0,
          "fabricSiteScore": 0,
          "fabricVnScore": 0,
          "fabsiteFcpScore": 0,
          "fabsiteInfraScore": 0,
          "fabsiteFsconnScore": 0,
          "vnExitScore": 0,
          "vnFcpScore": 0,
          "vnStatusScore": 0,
          "vnServiceScore": 0,
          "transitControlPlaneScore": 0,
          "transitServicesScore": 0,
          "tcpConnScore": 0,
          "bgpBgpSiteScore": 0,
          "vniStatusScore": 0,
          "pubsubTransitConnScore": 0,
          "bgpPeerInfraVnScore": 0,
          "internetAvailScore": 0,
          "bgpEvpnScore": 0,
          "lispTransitConnScore": 0,
          "ctsEnvDataDownloadScore": 0,
          "pubsubInfraVnScore": 0,
          "peerScore": 0,
          "bgpPeerScore": 0,
          "remoteInternetAvailScore": 0,
          "bgpTcpScore": 0,
          "pubsubSessionScore": 0,
          "aaaStatusScore": 0,
          "lispCpConnScore": 0,
          "bgpPubsubSiteScore": 0,
          "mcastScore": 0,
          "portChannelScore": 0
        },
        "aggregateAttributes": [
          {
            "name": "string",
            "function": "string",
            "value": 0
          }
        ]
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: application_visibility_network_devices_disable_cbar
short_description: Resource module for Application Visibility Network Devices Disable Cbar
description:
  - Manage operation create of the resource Application Visibility Network Devices Disable Cbar.
  - This API can be used to disable CBAR feature on multiple network devices.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  networkDeviceIds:
    description: List of network device ids where CBAR has to be disabled.
    elements: str
    type: list
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Application Policy DisableCBARFeatureOnMultipleNetworkDevices
    description: Complete reference of the DisableCBARFeatureOnMultipleNetworkDevices API.
    link: https://developer.cisco.com/docs/dna-center/#!disable-cbar-feature-on-multiple-network-devices
notes:
  - SDK Method used are
    application_policy.ApplicationPolicy.disable_c_b_a_r_feature_on_multiple_network_devices,
  - Paths used are
    post /dna/intent/api/v1/applicationVisibility/networkDevices/disableCbar,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.application_visibility_network_devices_disable_cbar:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    networkDeviceIds:
      - string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: fabrics_fabric_id_switch_wireless_setting_info
short_description: Information module for Fabrics Fabric Id Switch Wireless Setting
description:
  - Get all Fabrics Fabric Id Switch Wireless Setting. - > Get the SDA Wireless details from the switches on the fabric site
    that have wireless capability enabled. A maximum of two switches can have a wireless role in a fabric site.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  fabricId:
    description:
      - >
        FabricId path parameter. The 'fabricId' represents the Fabric ID of a particular Fabric Site. The
        'fabricId' can be obtained using the api /dna/intent/api/v1/sda/fabricSites. Example
        e290f1ee-6c54-4b01-90e6-d701748f0851.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Fabric Wireless GetSDAWirelessDetailsFromSwitches
    description: Complete reference of the GetSDAWirelessDetailsFromSwitches API.
    link: https://developer.cisco.com/docs/dna-center/#!get-sda-wireless-details-from-switches
notes:
  - SDK Method used are
    fabric_wireless.FabricWireless.get_sda_wireless_details_from_switches,
  - Paths used are
    get /dna/intent/api/v1/sda/fabrics/{fabricId}/switchWirelessSetting,
"""

EXAMPLES = r"""
---
- name: Get all Fabrics Fabric Id Switch Wireless Setting
  cisco.dnac.fabrics_fabric_id_switch_wireless_setting_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    fabricId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "enableWireless": true,
          "rollingApUpgrade": {
            "enableRollingApUpgrade": true,
            "apRebootPercentage": 0
          }
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: energy_clients_count_info
short_description: Information module for Energy Clients Count
description:
  - Get all Energy Clients Count.
  - Retrieves the total count of client devices that provide energy data, filtered according to query parameters.
  - For detailed information about the usage of the API, please refer to the Open API specification document
    https //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/
    CE_Cat_Center_Org-deviceEnergy_1.0-1.0.1-resolved.yaml.
version_added: '6.18.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description:
      - >
        StartTime query parameter. Start time from which API queries the data set related to the resource. It
        must be specified in UNIX epochtime in milliseconds. Value is inclusive. If `startTime` is not provided,
        API will default to one day before `endTime`.
    type: float
  endTime:
    description:
      - >
        EndTime query parameter. End time to which API queries the data set related to the resource. It must be
        specified in UNIX epochtime in milliseconds. Value is inclusive. If `endTime` is not provided, API will
        default to one day after `startTime`. If `startTime` is not provided either, API will default to current
        time.
    type: float
  id:
    description:
      - >
        Id query parameter. The list of Mac addresses (e.g., `54 9F C6 43 FF 80`). Examples `id=54 9F C6 43 FF
        80` (single device requested) `id=54 9F C6 43 FF 80&id=01 23 45 67 89 AB`.
    type: str
  siteId:
    description:
      - >
        SiteId query parameter. The UUID of the site. (Ex. `flooruuid`) Examples `?siteId=id1` (single id
        requested) `?siteId=id1&siteId=id2&siteId=id3` (multiple ids requested).
    type: str
  siteHierarchy:
    description:
      - >
        SiteHierarchy query parameter. The full hierarchical breakdown of the site tree starting from Global
        site name and ending with the specific site name. The Root site is named "Global" (Ex.
        `Global/AreaName/BuildingName/FloorName`) This field supports wildcard asterisk (`*`) character search
        support. E.g. `*/San*, */San, /San*` Examples `?siteHierarchy=Global/AreaName/BuildingName/FloorName`
        (single siteHierarchy requested) `?siteHierarchy=Global/AreaName/BuildingName/FloorName&siteHierarchy=Gl
        obal/AreaName2/BuildingName2/FloorName2` (multiple siteHierarchies requested).
    type: str
  siteHierarchyId:
    description:
      - >
        SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
        Global site UUID and ending with the specific site UUID. (Ex.
        `globalUuid/areaUuid/buildingUuid/floorUuid`) This field supports wildcard asterisk (`*`) character
        search support. E.g. `*uuid*, *uuid, uuid*` Examples
        `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteH
        ierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2
        /floorUuid2` (multiple siteHierarchyIds requested).
    type: str
  deviceCategory:
    description:
      - >
        DeviceCategory query parameter. The list of device deviceCategories. Examples
        `deviceCategory=AccessPoint` (single device family requested)
        `deviceCategory=AccessPoint&deviceCategory=OtherPOEDevice` (multiple device categories with comma
        separator).
    type: str
  deviceSubCategory:
    description:
      - >
        DeviceSubCategory query parameter. The list of device sub categories. Examples `deviceSubCategory=IP
        Phone 7821` (single sub category requested) `deviceSubCategory=IP Phone 7821&deviceSubCategory=IEEE PD`.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Clients CountClientsEnergy
    description: Complete reference of the CountClientsEnergy API.
    link: https://developer.cisco.com/docs/dna-center/#!count-clients-energy
notes:
  - SDK Method used are
    clients.Clients.count_clients_energy,
  - Paths used are
    get /dna/data/api/v1/energy/clients/count,
"""

EXAMPLES = r"""
---
- name: Get all Energy Clients Count
  cisco.dnac.energy_clients_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    id: string
    siteId: string
    siteHierarchy: string
    siteHierarchyId: string
    deviceCategory: string
    deviceSubCategory: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: dna_health_score_definitions_count_info
short_description: Information module for Dna Health Score Definitions Count
description:
  - Get all Dna Health Score Definitions Count.
  - Get the count of health score definitions based on provided filters.
  - Supported filters are id, name and overall health include status.
  - For detailed information about the usage of the API, please refer to the Open API specification document
    https //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-issueAndHealthDefinitions-1.0.0-resolved.yaml.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceType:
    description:
      - >
        DeviceType query parameter. These are the device families supported for health score definitions. If no
        input is made on device family, all device families are considered.
    type: str
  id:
    description:
      - >
        Id query parameter. The definition identifier. Examples id=015d9cba-4f53-4087-8317-7e49e5ffef46 (single
        entity id request) id=015d9cba-4f53-4087-8317-7e49e5ffef46&id=015d9cba-4f53-4087-8317-7e49e5ffef47
        (multiple ids in the query param).
    type: str
  includeForOverallHealth:
    description:
      - >
        IncludeForOverallHealth query parameter. The inclusion status of the issue definition, either true or
        false. True indicates that particular health metric is included in overall health computation, otherwise
        false. By default it's set to true.
    type: bool
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Devices GetTheCountOfHealthScoreDefinitionsBasedOnProvidedFilters
    description: Complete reference of the GetTheCountOfHealthScoreDefinitionsBasedOnProvidedFilters API.
    link: https://developer.cisco.com/docs/dna-center/#!get-the-count-of-health-score-definitions-based-on-provided-filters
notes:
  - SDK Method used are
    devices.Devices.get_the_count_of_health_score_definitions_based_on_provided_filters,
  - Paths used are
    get /dna/intent/api/v1/healthScoreDefinitions/count,
"""

EXAMPLES = r"""
---
- name: Get all Dna Health Score Definitions Count
  cisco.dnac.dna_health_score_definitions_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceType: string
    id: string
    includeForOverallHealth: true
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_controllers_ssid_details_info
short_description: Information module for Wireless Controllers Ssid Details
description:
  - Get all Wireless Controllers Ssid Details.
  - Retrieves all details of SSIDs associated with the specific Wireless Controller.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  networkDeviceId:
    description:
      - >
        NetworkDeviceId path parameter. Obtain the network device ID value by using the API call GET
        /dna/intent/api/v1/network-device/ip-address/${ipAddress}.
    type: str
  ssidName:
    description:
      - >
        SsidName query parameter. Employ this query parameter to obtain the details of the SSID corresponding to
        the provided SSID name.
    type: str
  adminStatus:
    description:
      - >
        AdminStatus query parameter. Utilize this query parameter to obtain the administrative status. A 'true'
        value signifies that the admin status of the SSID is enabled, while a 'false' value indicates that the
        admin status of the SSID is disabled.
    type: bool
  managed:
    description:
      - >
        Managed query parameter. If value is 'true' means SSIDs are configured through design.If the value is
        'false' means out of band configuration from the Wireless Controller.
    type: bool
  limit:
    description:
      - >
        Limit query parameter. The number of records to show for this page. Default is 500 if not specified.
        Maximum allowed limit is 500.
    type: int
  offset:
    description:
      - Offset query parameter. The first record to show for this page; the first record is numbered 1.
    type: int
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Wireless GetSSIDDetailsForSpecificWirelessController
    description: Complete reference of the GetSSIDDetailsForSpecificWirelessController API.
    link: https://developer.cisco.com/docs/dna-center/#!get-ssid-details-for-specific-wireless-controller
notes:
  - SDK Method used are
    wireless.Wireless.get_ssid_details_for_specific_wireless_controller,
  - Paths used are
    get /dna/intent/api/v1/wirelessControllers/{networkDeviceId}/ssidDetails,
"""

EXAMPLES = r"""
---
- name: Get all Wireless Controllers Ssid Details
  cisco.dnac.wireless_controllers_ssid_details_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    ssidName: string
    adminStatus: true
    managed: true
    limit: 0
    offset: 0
    networkDeviceId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "ssidName": "string",
          "wlanId": 0,
          "wlanProfileName": "string",
          "l2Security": "string",
          "l3Security": "string",
          "radioPolicy": "string",
          "adminStatus": true,
          "managed": true
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: lan_automation_port_channels_info
short_description: Information module for Lan Automation Port Channels
description:
  - Get all Lan Automation Port Channels.
  - Get Lan Automation Port Channels by id.
  - Returns a list of Port Channel between the LAN Automation associated devices.
  - This API retrieves Port Channel information using its ID.
version_added: '6.18.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  device1ManagementIPAddress:
    description:
      - Device1ManagementIPAddress query parameter. The management IP address of the device1.
    type: str
  device1Uuid:
    description:
      - Device1Uuid query parameter. Unique identifier for the network device1.
    type: str
  device2ManagementIPAddress:
    description:
      - Device2ManagementIPAddress query parameter. The management IP address of the device2.
    type: str
  device2Uuid:
    description:
      - Device2Uuid query parameter. Unique identifier for the network device2.
    type: str
  offset:
    description:
      - Offset query parameter. Starting record for pagination.
    type: int
  limit:
    description:
      - Limit query parameter. Maximum number of Port Channel to return.
    type: int
  id:
    description:
      - Id path parameter. ID of the port channel.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for LAN Automation GetPortChannelInformationById
    description: Complete reference of the GetPortChannelInformationById API.
    link: https://developer.cisco.com/docs/dna-center/#!get-port-channel-information-by-id
  - name: Cisco DNA Center documentation for LAN Automation GetPortChannels
    description: Complete reference of the GetPortChannels API.
    link: https://developer.cisco.com/docs/dna-center/#!get-port-channels
notes:
  - SDK Method used are
    lan_automation.LanAutomation.get_port_channel_information_by_id,
    lan_automation.LanAutomation.get_port_channels,
  - Paths used are
    get /dna/intent/api/v1/lanAutomation/portChannels,
    get /dna/intent/api/v1/lanAutomation/portChannels/{id},
"""

EXAMPLES = r"""
---
- name: Get all Lan Automation Port Channels
  cisco.dnac.lan_automation_port_channels_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    device1ManagementIPAddress: string
    device1Uuid: string
    device2ManagementIPAddress: string
    device2Uuid: string
    offset: 0
    limit: 0
  register: result
- name: Get Lan Automation Port Channels by id
  cisco.dnac.lan_automation_port_channels_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "device1ManagementIPAddress": "string",
          "device1Uuid": "string",
          "device2ManagementIPAddress": "string",
          "device2Uuid": "string",
          "device1PortChannelUuid": "string",
          "device1PortChannelNumber": 0,
          "device2PortChannelUuid": "string",
          "device2PortChannelNumber": 0,
          "portChannelMembers": [
            {
              "device1InterfaceUuid": "string",
              "device1Interface": "string",
              "device2InterfaceUuid": "string",
              "device2Interface": "string"
            }
          ]
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_intent_info
short_description: Information module for Network Devices Intent
description:
  - Get all Network Devices Intent.
  - Get Network Devices Intent by id. - > API to fetch the details of network device using the `id`. Use the `/dna/intent/api/v1/networkDevices/query`
    API for advanced filtering. The API supports views to fetch only the required fields. Refer features for more details.
    - > API to fetch the list of network devices using basic filters. Use the `/dna/intent/api/v1/networkDevices/query` API
    for advanced filtering. Refer features for more details. The API returns a paginated response based on 'limit' and 'offset'
    parameters, allowing up to 500 records per page. 'limit' specifies the number of records, and 'offset' sets the starting
    point using 1-based indexing. Use /dna/intent/api/v1/networkDevices/count API to get the total record count. For data
    sets over 500 records, make multiple calls, adjusting 'limit' and 'offset' to retrieve all records incrementally.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
      - Id query parameter. Network device Id.
    type: str
  managementAddress:
    description:
      - ManagementAddress query parameter. Management address of the network device.
    type: str
  serialNumber:
    description:
      - SerialNumber query parameter. Serial number of the network device.
    type: str
  family:
    description:
      - Family query parameter. Product family of the network device. For example, Switches, Routers, etc.
    type: str
  stackDevice:
    description:
      - StackDevice query parameter. Flag indicating if the device is a stack device.
    type: str
  role:
    description:
      - >
        Role query parameter. Role assigned to the network device. Available values BORDER_ROUTER, CORE,
        DISTRIBUTION, ACCESS, UNKNOWN.
    type: str
  status:
    description:
      - >
        Status query parameter. Inventory related status of the network device. Available values MANAGED,
        SYNC_NOT_STARTED, SYNC_INIT_FAILED, SYNC_PRECHECK_FAILED, SYNC_IN_PROGRESS, SYNC_INTERNAL_ERROR,
        SYNC_DISABLED, DELETING_DEVICE, UNDER_MAINTENANCE, QUARANTINED, UNASSOCIATED, UNREACHABLE, UNKNOWN.
        Refer features for more details.
    type: str
  reachabilityStatus:
    description:
      - >
        ReachabilityStatus query parameter. Reachability status of the network device. Available values
        REACHABLE, ONLY_PING_REACHABLE, UNREACHABLE, UNKNOWN. Refer features for more details.
    type: str
  managementState:
    description:
      - >
        ManagementState query parameter. The status of the network device's manageability. Available statuses
        are MANAGED, UNDER_MAINTENANCE, NEVER_MANAGED. Refer features for more details.
    type: str
  views:
    description:
      - >
        Views query parameter. The specific views being requested. This is an optional parameter which can be
        passed to get one or more of the network device data. If this is not provided, then it will default to
        BASIC views. If multiple views are provided, the response will contain the union of the views. Refer
        features for more details. Available values BASIC, RESYNC, USER_DEFINED_FIELDS.
    type: str
  limit:
    description:
      - Limit query parameter. The number of records to show for this page. Min 1, Max 500.
    type: int
  offset:
    description:
      - Offset query parameter. The first record to show for this page; the first record is numbered 1.
    type: int
  sortBy:
    description:
      - >
        SortBy query parameter. A property within the response to sort by. Available values id,
        managementAddress, dnsResolvedManagementIpAddress, hostname, macAddress, type, family, series,
        platformids, softwareType, softwareVersion, vendor, bootTime, role, roleSource, apEthernetMacAddress,
        apManagerInterfaceIpAddress, apWlcIpAddress, deviceSupportLevel, reachabilityFailureReason,
        resyncStartTime, resyncEndTime, resyncReasons, pendingResyncRequestCount, pendingResyncRequestReasons,
        resyncIntervalSource, resyncIntervalMinutes.
    type: str
  order:
    description:
      - Order query parameter. Whether ascending or descending order should be used to sort the response.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Devices GetDetailsOfASingleNetworkDevice
    description: Complete reference of the GetDetailsOfASingleNetworkDevice API.
    link: https://developer.cisco.com/docs/dna-center/#!get-details-of-a-single-network-device
  - name: Cisco DNA Center documentation for Devices RetrieveNetworkDevices
    description: Complete reference of the RetrieveNetworkDevices API.
    link: https://developer.cisco.com/docs/dna-center/#!retrieve-network-devices
notes:
  - SDK Method used are
    devices.Devices.get_details_of_a_single_network_device,
    devices.Devices.retrieve_network_devices,
  - Paths used are
    get /dna/intent/api/v1/networkDevices,
    get /dna/intent/api/v1/networkDevices/{id},
"""

EXAMPLES = r"""
---
- name: Get all Network Devices Intent
  cisco.dnac.network_devices_intent_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
    managementAddress: string
    serialNumber: string
    family: string
    stackDevice: string
    role: string
    status: string
    reachabilityStatus: string
    managementState: string
    views: string
    limit: 0
    offset: 0
    sortBy: string
    order: string
  register: result
- name: Get Network Devices Intent by id
  cisco.dnac.network_devices_intent_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    views: string
    id: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "managementAddress": "string",
          "dnsResolvedManagementIpAddress": "string",
          "hostname": "string",
          "macAddress": "string",
          "serialNumbers": [
            "string"
          ],
          "type": "string",
          "family": "string",
          "series": "string",
          "status": "string",
          "platformIds": [
            "string"
          ],
          "softwareType": "string",
          "softwareVersion": "string",
          "vendor": "string",
          "stackDevice": true,
          "bootTime": 0,
          "role": "string",
          "roleSource": "string",
          "apEthernetMacAddress": "string",
          "apManagerInterfaceIpAddress": "string",
          "apWlcIpAddress": "string",
          "deviceSupportLevel": "string",
          "snmpLocation": "string",
          "snmpContact": "string",
          "reachabilityStatus": "string",
          "reachabilityFailureReason": "string",
          "managementState": "string",
          "lastSuccessfulResyncReasons": [
            "string"
          ],
          "resyncStartTime": 0,
          "resyncEndTime": 0,
          "resyncReasons": [
            "string"
          ],
          "resyncRequestedByApps": [
            "string"
          ],
          "pendingResyncRequestCount": 0,
          "pendingResyncRequestReasons": [
            "string"
          ],
          "resyncIntervalSource": "string",
          "resyncIntervalMinutes": 0,
          "errorCode": "string",
          "errorDescription": "string",
          "userDefinedFields": {}
        }
      ],
      "version": "string"
    }
"""



"""Ansible-rulebook event source plugin for Azure Service Bus.

This module provides asynchronous consumers for Azure Service Bus to feed
events into Ansible EDA rulebooks.
"""

import asyncio
import contextlib
import json
from typing import Any

from azure.servicebus.aio import ServiceBusClient

DOCUMENTATION = r"""
---
short_description: Receive events from an Azure service bus.
description:
  - An ansible-rulebook event source module for receiving events from an Azure service bus.
  - In order to get the service bus and the connection string, refer to
    https://learn.microsoft.com/en-us/azure/service-bus-messaging/service-bus-python-how-to-use-queues?tabs=passwordless
options:
  conn_str:
    description:
      - The connection string to connect to the Azure service bus.
    type: str
    required: true
  queue_name:
    description:
      - The name of the queue to pull messages from.
    type: str
    required: true
  logging_enable:
    description:
      - Whether to turn on logging.
    type: bool
    default: true
"""

EXAMPLES = r"""
- azure.azcollection.azure_service_bus:
    conn_str: "{{connection_str}}"
    queue_name: "{{queue_name}}"
"""


async def receive_events(
    queue: asyncio.Queue[Any],
    args: dict[str, Any],  # pylint: disable=W0621
) -> None:
    """Receive events from service bus asynchronously."""
    servicebus_client = ServiceBusClient.from_connection_string(
        conn_str=args["conn_str"],
        logging_enable=bool(args.get("logging_enable", True)),
    )

    async with servicebus_client:
        receiver = servicebus_client.get_queue_receiver(queue_name=args["queue_name"])
        async with receiver:
            async for msg in receiver:
                meta = {"message_id": msg.message_id}
                body = str(msg)
                with contextlib.suppress(json.JSONDecodeError):
                    body = json.loads(body)

                await queue.put({"body": body, "meta": meta})
                await receiver.complete_message(msg)


async def main(
    queue: asyncio.Queue[Any],
    args: dict[str, Any],  # pylint: disable=W0621
) -> None:
    """Receive events from service bus in a loop."""
    await receive_events(queue, args)


if __name__ == "__main__":
    # MockQueue if running directly.

    class MockQueue(asyncio.Queue[Any]):
        """A fake queue."""

        async def put(self: "MockQueue", event: dict[str, Any]) -> None:
            """Print the event."""
            print(event)  # noqa: T201

    args = {
        "conn_str": "Endpoint=sb://foo.servicebus.windows.net/",
        "queue_name": "eda-queue",
    }
    asyncio.run(main(MockQueue(), args))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_applications_id_trend_analytics
short_description: Resource module for Network Applications Id Trend Analytics
description:
  - Manage operation create of the resource Network Applications Id Trend Analytics. - > Retrieves the trend analytics of
    applications experience data to specific network application for the specified time range. The data will be grouped based
    on the given trend time interval. This API facilitates obtaining consolidated insights into the performance and status
    of the network applications over the specified start and end time. If startTime and endTime are not provided, the API
    defaults to the last 24 hours.`siteId` and `trendInterval` are mandatory. `siteId` must be a site UUID of a building.For
    detailed information about the usage of the API, please refer to the Open API specification document - https //github.com/cisco-en-
    programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org- NetworkApplications-1.0.1-resolved.yaml.
version_added: '6.18.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Network Applications Id Trend Analytics's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Network Applications Id Trend Analytics's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  id:
    description: Id path parameter. Id is the network application name.
    type: str
  page:
    description: Network Applications Id Trend Analytics's page.
    suboptions:
      cursor:
        description: Cursor.
        type: str
      limit:
        description: Limit.
        type: int
      timeSortOrder:
        description: Time Sort Order.
        type: str
    type: dict
  siteIds:
    description: Site Ids.
    elements: str
    type: list
  startTime:
    description: Start Time.
    type: int
  trendInterval:
    description: Trend Interval.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Applications RetrievesTheTrendAnalyticsRelatedToSpecificNetworkApplication
    description: Complete reference of the RetrievesTheTrendAnalyticsRelatedToSpecificNetworkApplication API.
    link: https://developer.cisco.com/docs/dna-center/#!retrieves-the-trend-analytics-related-to-specific-network-application
notes:
  - SDK Method used are
    applications.Applications.retrieves_the_trend_analytics_related_to_specific_network_application,
  - Paths used are
    post /dna/data/api/v1/networkApplications/{id}/trendAnalytics,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.network_applications_id_trend_analytics:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
      - function: string
        name: string
    attributes:
      - string
    endTime: 0
    filters:
      - key: string
        operator: string
        value: string
    headers: '{{my_headers | from_json}}'
    id: string
    page:
      cursor: string
      limit: 0
      timeSortOrder: string
    siteIds:
      - string
    startTime: 0
    trendInterval: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "timestamp": 0,
          "attributes": [
            {
              "name": "string",
              "value": "string"
            }
          ],
          "aggregateAttributes": [
            {
              "name": "string",
              "function": "string",
              "value": 0
            }
          ]
        }
      ],
      "page": {
        "limit": 0,
        "cursor": "string",
        "count": 0,
        "timeSortOrder": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: system_settings_ap_pnp_location_info
short_description: Information module for System Settings Ap Pnp Location
description:
  - Get all System Settings Ap Pnp Location. - > Retrieve the current AP PnP Location setting from the system.Once the AP
    PnP Location Setting is enabled, the Access Point's assigned Site name will be configured as the AP Location during the
    PnP Claim process. This applies only during the PnP onboarding process and not during any subsequent provisioning dayN
    .
version_added: '6.18.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Wireless GetAPPnPLocationSetting
    description: Complete reference of the GetAPPnPLocationSetting API.
    link: https://developer.cisco.com/docs/dna-center/#!get-ap-pn-p-location-setting
notes:
  - SDK Method used are
    wireless.Wireless.get_ap_pnp_location_setting,
  - Paths used are
    get /dna/intent/api/v1/systemSettings/apPnpLocation,
"""

EXAMPLES = r"""
---
- name: Get all System Settings Ap Pnp Location
  cisco.dnac.system_settings_ap_pnp_location_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: str
  sample: >
    "string"
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: connection_mode_setting
short_description: Resource module for Connection Mode Setting
description:
  - Manage operation update of the resource Connection Mode Setting.
  - Update Cisco Smart Software Manager CSSM connection mode for the system.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  connectionMode:
    description: The CSSM connection modes of Catalyst Center are DIRECT, ON_PREMISE and SMART_PROXY.
    type: str
  parameters:
    description: Connection Mode Setting's parameters.
    suboptions:
      clientId:
        description: On-premise CSSM client id.
        type: str
      clientSecret:
        description: On-premise CSSM client secret.
        type: str
      onPremiseHost:
        description: On-premise CSSM hostname or IP address.
        type: str
      smartAccountName:
        description: On-premise CSSM local smart account name.
        type: str
    type: dict
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Licenses UpdateCSSMConnectionMode
    description: Complete reference of the UpdateCSSMConnectionMode API.
    link: https://developer.cisco.com/docs/dna-center/#!update-cssm-connection-mode
notes:
  - SDK Method used are
    licenses.Licenses.update_c_s_s_m_connection_mode,
  - Paths used are
    put /dna/intent/api/v1/connectionModeSetting,
"""

EXAMPLES = r"""
---
- name: Update all
  cisco.dnac.connection_mode_setting:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    connectionMode: string
    parameters:
      clientId: string
      clientSecret: string
      onPremiseHost: string
      smartAccountName: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "count": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ipam_site_ip_address_pools_info
short_description: Information module for Ipam Site Ip Address Pools
description:
  - Get all Ipam Site Ip Address Pools.
  - Get Ipam Site Ip Address Pools by id.
  - Retrieves IP address subpools, which reserve address space from a global pool or global pools . - > Retrieves an IP address
    subpool, which reserves address space from a global pool or global pools for a particular site.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  offset:
    description:
      - Offset query parameter. The first record to show for this page; the first record is numbered 1.
    type: int
  limit:
    description:
      - Limit query parameter. The number of records to show for this page; the minimum is 1, and the maximum is 500.
    type: int
  sortBy:
    description:
      - SortBy query parameter. A property within the response to sort by.
    type: str
  order:
    description:
      - Order query parameter. Whether ascending or descending order should be used to sort the response.
    type: str
  siteId:
    description:
      - >
        SiteId query parameter. The `id` of the site for which to retrieve IP address subpools. Only subpools
        whose `siteId` exactly matches will be fetched, parent or child site matches will not be included.
    type: str
  id:
    description:
      - Id path parameter. The `id` of the IP address subpool to retrieve.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Network Settings RetrievesAnIPAddressSubpool
    description: Complete reference of the RetrievesAnIPAddressSubpool API.
    link: https://developer.cisco.com/docs/dna-center/#!retrieves-an-ip-address-subpool
  - name: Cisco DNA Center documentation for Network Settings RetrievesIPAddressSubpools
    description: Complete reference of the RetrievesIPAddressSubpools API.
    link: https://developer.cisco.com/docs/dna-center/#!retrieves-ip-address-subpools
notes:
  - SDK Method used are
    network_settings.NetworkSettings.retrieves_an_ip_address_subpool,
    network_settings.NetworkSettings.retrieves_ip_address_subpools,
  - Paths used are
    get /dna/intent/api/v1/ipam/siteIpAddressPools,
    get /dna/intent/api/v1/ipam/siteIpAddressPools/{id},
"""

EXAMPLES = r"""
---
- name: Get all Ipam Site Ip Address Pools
  cisco.dnac.ipam_site_ip_address_pools_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
    sortBy: string
    order: string
    siteId: string
  register: result
- name: Get Ipam Site Ip Address Pools by id
  cisco.dnac.ipam_site_ip_address_pools_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "ipV4AddressSpace": {
          "subnet": "string",
          "prefixLength": 0,
          "gatewayIpAddress": "string",
          "dhcpServers": [
            "string"
          ],
          "dnsServers": [
            "string"
          ],
          "totalAddresses": "string",
          "unassignableAddresses": "string",
          "assignedAddresses": "string",
          "defaultAssignedAddresses": "string",
          "slaacSupport": true,
          "globalPoolId": "string",
          "overlapping": true,
          "addressSpaceId": "string"
        },
        "ipV6AddressSpace": {
          "subnet": "string",
          "prefixLength": 0,
          "gatewayIpAddress": "string",
          "dhcpServers": [
            "string"
          ],
          "dnsServers": [
            "string"
          ],
          "totalAddresses": "string",
          "unassignableAddresses": "string",
          "assignedAddresses": "string",
          "defaultAssignedAddresses": "string",
          "slaacSupport": true,
          "globalPoolId": "string",
          "overlapping": true,
          "addressSpaceId": "string"
        },
        "name": "string",
        "poolType": "string",
        "siteId": "string",
        "siteName": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: system_performance_historical_info
short_description: Information module for System Performance Historical
description:
  - Get all System Performance Historical. - > Retrieves hourly data of cluster key performance indicators KPIs , like CPU
    utilization, memory utilization or network rates for the past 24 hours. Query parameters 'startTime' and 'endTime' are
    no longer supported.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  kpi:
    description:
      - Kpi query parameter. Fetch historical data for this kpi. Valid values cpu,memory,network.
    type: str
  startTime:
    description:
      - >
        StartTime query parameter. In this release this field has been deprecated and no longer supported. This
        is restricted to a 24-hour time range with hourly samples, regardless of the provided epoch time. For
        example, if any epoch time is specified for this field, it will be ignored and 24 hours before the
        current time will be considered.
    type: float
  endTime:
    description:
      - >
        EndTime query parameter. In this release this field has been deprecated and no longer supported. This is
        restricted to a 24-hour time range with hourly samples, regardless of the provided epoch time. For
        example, if any epoch time is specified for this field, it will be ignored and the current time will be
        considered.
    type: float
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Health and Performance SystemPerformanceHistoricalAPI
    description: Complete reference of the SystemPerformanceHistoricalAPI API.
    link: https://developer.cisco.com/docs/dna-center/#!system-performance-historical-api
notes:
  - SDK Method used are
    health_and_performance.HealthAndPerformance.system_performance_historical,
  - Paths used are
    get /dna/intent/api/v1/diagnostics/system/performance/history,
"""

EXAMPLES = r"""
---
- name: Get all System Performance Historical
  cisco.dnac.system_performance_historical_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    kpi: string
    startTime: 0
    endTime: 0
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "hostName": "string",
      "version": "string",
      "kpis": {
        "cpu": {
          "units": "string",
          "utilization": "string"
        },
        "memory": {
          "units": "string",
          "utilization": "string"
        },
        "network tx_rate": {
          "units": "string",
          "utilization": "string"
        },
        "network rx_rate": {
          "units": "string",
          "utilization": "string"
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_members_associations_query
short_description: Resource module for Network Devices Members Associations Query
description:
  - Manage operation create of the resource Network Devices Members Associations Query. - > Fetches the tags associated with
    the given network device `ids`. Devices that don't have any tags associated will not be included in the response. A tag
    is a user-defined or system-defined construct to group resources. When a device is tagged, it is called a member of the
    tag. `ids` can be fetched via `/dna/intent/api/v1/network-device` API.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  ids:
    description: List of member ids (network device or interface), maximum 500 ids can be passed.
    elements: str
    type: list
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Tag QueryTheTagsAssociatedWithNetworkDevices
    description: Complete reference of the QueryTheTagsAssociatedWithNetworkDevices API.
    link: https://developer.cisco.com/docs/dna-center/#!query-the-tags-associated-with-network-devices
notes:
  - SDK Method used are
    tag.Tag.query_the_tags_associated_with_network_devices,
  - Paths used are
    post /dna/intent/api/v1/tags/networkDevices/membersAssociations/query,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.network_devices_members_associations_query:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    ids:
      - string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "tags": [
            {
              "id": "string",
              "name": "string"
            }
          ]
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: tags_network_devices_members_associations_count_info
short_description: Information module for Tags Network Devices Members Associations Count
description:
  - Get all Tags Network Devices Members Associations Count. - > Fetches the count of network devices that are associated
    with at least one tag. A tag is a user-defined or system-defined construct to group resources. When a device is tagged,
    it is called a member of the tag.
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Tag RetrieveTheCountOfNetworkDevicesThatAreAssociatedWithAtLeastOneTag
    description: Complete reference of the RetrieveTheCountOfNetworkDevicesThatAreAssociatedWithAtLeastOneTag API.
    link: https://developer.cisco.com/docs/dna-center/#!retrieve-the-count-of-network-devices-that-are-associated-with-at-least-one-tag
notes:
  - SDK Method used are
    tag.Tag.retrieve_the_count_of_network_devices_that_are_associated_with_at_least_one_tag,
  - Paths used are
    get /dna/intent/api/v1/tags/networkDevices/membersAssociations/count,
"""

EXAMPLES = r"""
---
- name: Get all Tags Network Devices Members Associations Count
  cisco.dnac.tags_network_devices_members_associations_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: icap_settings_configuration_models_id_delete_deploy
short_description: Resource module for Icap Settings Configuration Models Id Delete Deploy
description:
  - Manage operation create of the resource Icap Settings Configuration Models Id Delete Deploy. - > Creates a ICAP configuration
    intent to remove the ICAP RFSTATS or ANOMALY configuration from the device. The task has not been applied to the device
    yet. Subsequent preview-approve workflow APIs must be used to complete the preview-approve process. The path parameter
    'id' can be retrieved from **GET /dna/intent/api/v1/icapSettings** API. For detailed information about the usage of the
    API, please refer to the Open API specification document - https //github.com/cisco-en-programmability/catalyst-center-api-
    specs/blob/main/Assurance/CE_Cat_Center_Org-ICAP_APIs-1.0.0-resolved.yaml.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. A unique ID of the deployed ICAP object, which can be obtained from **GET /dna/intent/api/v1/icapSettings**.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Sensors CreatesAICAPConfigurationWorkflowForICAPIntentToRemoveTheICAPConfigurationOnTheDevice
    description: Complete reference of the CreatesAICAPConfigurationWorkflowForICAPIntentToRemoveTheICAPConfigurationOnTheDevice
      API.
    link: https://developer.cisco.com/docs/dna-center/#!creates-aicap-configuration-workflow-for-icap-intent-to-remove-the-icap-configuration-on-the-device
notes:
  - SDK Method used are
    sensors.Sensors.creates_ai_cap_configuration_workflow_for_i_capintent_to_remove_the_i_cap_configuration_on_the_device,
  - Paths used are
    post /dna/intent/api/v1/icapSettings/configurationModels/{id}/deleteDeploy,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.icap_settings_configuration_models_id_delete_deploy:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    id: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2024, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function

__metaclass__ = type
__author__ = (
    "Abinash Mishra, Rugvedi Kapse, Madhan Sankaranarayanan, Sonali Deepthi Kesali"
)
DOCUMENTATION = r"""
---
module: device_configs_backup_workflow_manager
short_description: Device Configs Backup module for
  taking configuration backups of reachable devices
  in the Cisco Catalyst Center.
description:
  - Manage operation related to taking the backup of
    running config, static config and vlan.dat.bat
version_added: "6.14.0"
extends_documentation_fragment:
  - cisco.dnac.workflow_manager_params
author: Abinash Mishra (@abimishr) Rugvedi Kapse (@rukapse)
  Madhan Sankaranarayanan (@madhansansel) Sonali Deepthi
  Kesali (@skesali)
options:
  config_verify:
    description: Set to True to verify the Cisco Catalyst
      Center config after applying the playbook config.
    type: bool
    default: false
  state:
    description: The state of Cisco Catalyst Center
      after module completion.
    type: str
    choices: [merged]
    default: merged
  config:
    description:
      - List of details of the devices for which configuration
        backups need to be taken.
      - At least one parameter from the suboptions must
        be provided in the config.
      - When providing multiple parameters at once (excluding
        "site_list"), all the filters will be applied
        together in an AND operation. This means that
        only the devices matching all the specified
        criteria will be included in the configuration
        backup operation. For example, if both `hostname`
        and `device_type` are provided, only devices
        matching both the specified hostname and device
        type will be selected.
      - Note - Once all devices matching the parameters
        are retrieved, any device that is not reachable
        or is an Access Point (AP) will be skipped.
    type: list
    elements: dict
    required: true
    suboptions:
      hostname_list:
        description:
          - List of hostnames of the devices for which
            a configuration backup is to be taken.
          - The hostnames must be identical to those
            displayed under the inventory section in
            the Cisco Catalyst Center GUI.
          - For example - ["DC-T-9300.cisco.local",
            "NY-BN-9300.cisco.local"]
        type: list
        elements: str
      ip_address_list:
        description:
          - List of IP addresses of the devices for
            which configuration backups need to be taken.
          - The IP addresses should match those displayed
            in the inventory GUI of the Cisco Catalyst
            Center, specifically the management IP addresses
            of the devices.
          - For example - ["204.1.2.2", "204.1.2.5",
            "204.1.2.4"]
        type: list
        elements: str
      site_list:
        description:
          - Specifies a list of sites. The module takes
            a configuration backup of all devices located
            within the specified site(s).
          - Each site should be represented as a string
            value that indicates the complete hierarchical
            path of the site.
          - For example - ["Global/USA/San Francisco/Building_2/floor_1",
            "Global/USA/New York/Building_3/floor_2"]
          - Note -  When additional parameters are provided
            along with `site_list`, the operation will
            include all devices in the specified site(s)
            and any devices matching the additional
            criteria (excluding `site_list`). In other
            words, the operation will be performed on
            devices within the site(s) and those that
            meet the additional criteria.
        type: list
        elements: str
      mac_address_list:
        description:
          - Specifies list of MAC addresses of the devices
            for which configuration backups are to be
            taken.
        type: list
        elements: str
      serial_number_list:
        description:
          - Specifies the list of serial numbers of
            the devices for which configuration backups
            need to be taken.
          - For example - ["FCW2225C020", "FJB2334D06N",
            "FJC2327U0S2", "FJC2721271T"]
        type: list
        elements: str
      family_list:
        description:
          - Specifies list of families for which device
            configuration backups need to be taken.
          - For example - ["Switches and Hubs", "Routers"]
        type: list
        elements: str
      type_list:
        description:
          - Specifies the list of types of device(s)
            from a specific device family for which
            configuration backups need to be taken.
          - For example - ["Cisco Catalyst 9300 Switch",
            "Cisco Catalyst 9500 Switch"]
        type: list
        elements: str
      series_list:
        description:
          - Specifies the list of series of the device(s)
            for a specific device type for which configuration
            backups need to be taken.
          - For example - ["Cisco Catalyst 9300 Series
            Switches"]
        type: list
        elements: str
      collection_status_list:
        description:
          - Specifies the list of collection status
            of the device(s) as displayed in the inventory
            GUI of the Cisco Catalyst Center.
          - For example - ["Managed"]
        type: list
        elements: str
      file_path:
        description:
          - The location or directory where the configuration
            backups need to be exported on the local
            system.
          - If the "file_path" is not provided, the
            backup file(s) will be stored in a directory
            named "tmp" in the same directory as the
            playbook.
        type: str
        default: tmp
      file_password:
        description:
          - Optional file password for zipping and unzipping
            the config file.
          - If not provided, a secure password will be auto-generated.
          - Password must meet the following criteria
            - - Minimum password length is 8 - It should
            contain atleast one lower case letter, one
            uppercase letter, - one digit - one special
            characters from -=\\\\\\\\;,./~!@$%^&*()_+{}[]|:?"
        type: str
      unzip_backup:
        description:
          - Determines whether the downloaded backup
            file should be unzipped after download.
          - If set to True, the backup file will be
            extracted to the specified directory.
          - If set to False, the file will remain in
            its zipped state.
        type: bool
        default: true
      config_file_types:
        description:
          - Specifies the list of configuration file types to be downloaded for each device.
          - If this parameter is not specified, all available configuration types
            will be downloaded for the selected devices by default.
          - This parameter is available starting from Cisco Catalyst Center version 2.3.7.9 and later.
          - For Example - ["VLAN", "STARTUPCONFIG", "RUNNINGCONFIG"]
        type: list
        elements: str
        choices:
          - ALL
          - VLAN
          - STARTUPCONFIG
          - RUNNINGCONFIG
        default: ["ALL"]
requirements:
  - dnacentersdk == 2.9.2
  - python >= 3.5
notes:
  - SDK Methods used are sites.Sites.get_site Site_design.Site_design.get_sites
    sites.Sites.get_membership site_design.Site_design.get_site_assigned_network_devices
    devices.Devices.get_device_list devices.Devices.get_device_by_id
    configuration_archive.ConfigurationsArchive.export_device_configurations
    configuration_archive.ConfigurationsArchive.download_unmaskedraw_device_configuration_as_zip
    configuration_archive.ConfigurationsArchive.download_masked_device_configuration
    configuration_archive.ConfigurationsArchive.get_network_device_configuration_file_details
    file.Files.download_a_file_by_fileid
  - Paths used are
    get /dna/intent/api/v1/site get /dna/intent/api/v1/membership/${siteId}
    get /dna/intent/api/v1/network-device post /dna/intent/api/v1/network-device-archive/cleartext
    get /dna/intent/api/v1/file/${fileId} get /dna/intent/api/v1/networkDevices/assignedToSite
    get /dna/intent/api/v1/sites get /dna/intent/api/v1/network-device/${id}
    get /dna/intent/api/v1/network-device get /dna/intent/api/v1/networkDeviceConfigFiles
    post /dna/intent/api/v1/network-device post dna/intent/api/v1/networkDeviceConfigFiles/${id}/downloadMasked
    post /dna/intent/api/v1/network-device post dna/intent/api/v1/networkDeviceConfigFiles/${id}/downloadUnMasked
"""
EXAMPLES = r"""
---
- name: Take backup of all devices in the Cisco Catalyst
    Center
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - file_password: qsaA12!asdasd
- name: Take backup of device(s) using hostname(s)
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - hostname_list: ["DC-T-9300.cisco.local", "NY-BN-9300.cisco.local"]
        file_path: backup
        unzip_backup: false
- name: Take backup of device(s) using hostname(s) and
    provide file password
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - hostname_list: ["DC-T-9300.cisco.local"]
        file_path: backup
        file_password: qsaA12!asdasd
        unzip_backup: true
- name: Take backup of all devices in a site(s)
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - site_list: ["Global/USA/RTP/BLD10", "Global/USA/New
            York/BLDNYC/FLOOR1"]
        file_path: backup
- name: Take backup of device(s) using IP Address List
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - ip_address_list: ["204.1.2.5", "204.1.2.4", "204.1.2.2"]
        file_path: backup
- name: Take backup of device(s) using MAC Address List
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - mac_address_list: ["d4:ad:bd:c1:67:00", " 00:b6:70:32:b8:00", "0c:75:bd:42:c3:80", "90:88:55:07:59:00"]
        file_path: backup
        unzip_backup: false
- name: Take backup of device(s) using Serial Number
    List
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - serial_number_list: ["FCW2225C020", "FJB2334D06N", "FJC2327U0S2", "FJC2721271T"]
        file_path: backup
- name: Take backup of device(s) using Family List
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - family_list: ["Switches and Hubs", "Routers"]
        file_path: backup
        unzip_backup: true
- name: Take backup of device(s) using Device Family
    Type List
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - type_list: ["Cisco Catalyst 9300 Switch"]
        file_path: backup
        unzip_backup: false
- name: Take backup of device(s) using Device Series
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - series_list: ["Cisco Catalyst 9300 Series Switches"]
        file_path: backup
- name: Take backup of devices with certain Collection
    Status
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - collection_status_list: ["Managed"]
        file_path: backup
- name: Take backup of device(s) in a site and also
    that meet other parameters
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - site_list: ["Global"]
        family_list: ["Switches and Hubs"]
        series_list: ["Cisco Catalyst 9300 Series Switches"]
        ip_address_list: ["204.1.2.5"]
        file_path: backup
        unzip_backup: false
- name: Take backup of device(s) using specified configuration file types
  cisco.dnac.device_configs_backup_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log: true
    dnac_log_level: "{{dnac_log_level}}"
    state: merged
    config:
      - site_list: ["Global"]
        family_list: ["Switches and Hubs"]
        ip_address_list: ["204.1.2.5"]
        file_path: backup
        unzip_backup: false
        config_file_types:
          - ALL
          - VLAN
          - STARTUPCONFIG
          - RUNNINGCONFIG
"""
RETURN = r"""
# Case_1: Successful creation and exportation of device configs
response_1:
  description: A dictionary with  with the response returned by the Cisco Catalyst Center Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response":
        {
          "response": String,
          "version": String
        },
      "msg": String
    }
# Case_2: Error while taking a device_configs_backup
response_2:
  description: A string with the response returned by the Cisco Catalyst Center Python SDK
  returned: always
  type: list
  sample: >
    {
      "response": [],
      "msg": String
    }
"""
# common approach when a module relies on optional dependencies that are not available during the validation process.
try:
    import pyzipper

    HAS_PYZIPPER = True
except ImportError:
    HAS_PYZIPPER = False
    pyzipper = None

try:
    import pathlib

    HAS_PATHLIB = True
except ImportError:
    HAS_PATHLIB = False
    pathlib = None

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.dnac.plugins.module_utils.dnac import (
    DnacBase,
    validate_list_of_dicts,
)
from io import BytesIO
import random
import string
import re
import time
import datetime
import os


class DeviceConfigsBackup(DnacBase):
    """
    Class containing member attributes for device_configs_backup workflow_manager module
    """

    def __init__(self, module):
        super().__init__(module)
        self.supported_states = ["merged"]
        self.skipped_devices_list = []

    def validate_input(self):
        """
        Validate the fields provided in the playbook.  Checks the
        configuration provided in the playbook against a predefined
        specification to ensure it adheres to the expected structure
        and data types.
        Parameters:
          - self: The instance of the class containing the "config" attribute
                  to be validated.
        Returns:
          The method returns an instance of the class with updated attributes:
          - self.msg: A message describing the validation result.
          - self.status: The status of the validation (either "success" or "failed").
          - self.validated_config: If successful, a validated version of the
                                   "config" parameter.
        """
        # Check if Pyzipper is installed
        if HAS_PYZIPPER is False:
            msg = "Pyzipper is not installed. Please install it using 'pip install pyzipper' command"
            self.log(msg, "CRITICAL")
            self.module.fail_json(msg=msg)

        # Check if Pathlib is installed
        if HAS_PATHLIB is False:
            msg = "Pathlib is not installed. Please install it using 'pip install pathlib' command"
            self.log(msg, "CRITICAL")
            self.module.fail_json(msg=msg)

        # Check if the config is provided in the playbook
        if not self.config:
            self.msg = "config not available in playbook for validattion"
            self.status = "success"
            return self

        # Define the specification for device configuration backup parameters
        device_configs_backup_spec = {
            "hostname_list": {"type": "list", "elements": "str", "required": False},
            "site_list": {"type": "list", "elements": "str", "required": False},
            "ip_address_list": {"type": "list", "elements": "str", "required": False},
            "mac_address_list": {"type": "list", "elements": "str", "required": False},
            "serial_number_list": {
                "type": "list",
                "elements": "str",
                "required": False,
            },
            "family_list": {"type": "list", "elements": "str", "required": False},
            "type_list": {"type": "list", "elements": "str", "required": False},
            "series_list": {"type": "list", "elements": "str", "required": False},
            "collection_status_list": {
                "type": "list",
                "elements": "str",
                "required": False,
            },
            "file_path": {"type": "str", "required": False, "default": "tmp"},
            "file_password": {"type": "str", "required": False},
            "unzip_backup": {"type": "bool", "required": False, "default": True},
            "config_file_types": {"type": "list", "elements": "str", "required": False}
        }

        # Validate device_configs_backup params
        valid_device_configs_backup, invalid_params = validate_list_of_dicts(
            self.config, device_configs_backup_spec
        )

        # Check if there are any invalid parameters
        if invalid_params:
            self.msg = "Invalid parameters in playbook: {0}".format(
                "\n".join(invalid_params)
            )
            self.log(str(self.msg), "ERROR")
            self.status = "failed"
            return self

        # If validation is successful, update the result
        self.validated_config = valid_device_configs_backup
        self.msg = "Successfully validated playbook configuration parameters using 'validate_input': {0}".format(
            str(valid_device_configs_backup)
        )
        self.status = "success"
        return self

    def get_device_list_params(self, config):
        """
        Generates a dictionary of device parameters for querying Cisco Catalyst Center.

        Parameters:
            config (dict): A dictionary containing device filter criteria.

        Returns:
            dict: A dictionary mapping internal parameter names to their corresponding values from the config.

        Description:
            This method takes a configuration dictionary containing various device filter criteria and maps them to the internal parameter
            names required by Cisco Catalyst Center.
            It returns a dictionary of these mapped parameters which can be used to query devices based on the provided filters.
        """
        self.log("Starting get_device_list_params function.", "DEBUG")
        self.log("Input config: {}".format(config), "DEBUG")

        # Initialize an empty dictionary to store the mapped parameters
        get_device_list_params = {}

        # Mapping from input parameter names to API Specific parameter names
        parameters_list = {
            "hostname_list": "hostname",
            "ip_address_list": "management_ip_address",
            "mac_address_list": "mac_address",
            "serial_number_list": "serial_number",
            "family_list": "family",
            "type_list": "type",
            "series_list": "series",
            "collection_status_list": "collection_status",
        }

        self.log("Parameter mapping list: {}".format(parameters_list), "DEBUG")

        # Iterate over the parameters and add them to the result dictionary if present in the config
        for parameter, parameter_name in parameters_list.items():
            param_value = config.get(parameter)
            if param_value:
                self.log("Parameter '{}' found in config with value: {}".format(parameter, param_value), "DEBUG")

                # If the parameter is serial_number_list, modify each serial number
                if parameter == "serial_number_list":
                    # Handle case where serial numbers are provided as a single comma-separated string
                    all_serial_numbers = []
                    for serial_item in param_value:
                        # Split if there are multiple serial numbers in one string
                        split_serials = serial_item.split(",")
                        for serial in split_serials:
                            serial_number = serial.strip()
                            all_serial_numbers.append(serial_number)

                    # Add wildcard prefix and suffix
                    serial_numbers_with_wildcards = []
                    for serial_number in all_serial_numbers:
                        serial_with_wildcard = ".*" + serial_number + ".*"
                        serial_numbers_with_wildcards.append(serial_with_wildcard)

                    get_device_list_params[parameter_name] = serial_numbers_with_wildcards
                    self.log("Modified serial_number_list with wildcards: {}".format(serial_numbers_with_wildcards), "DEBUG")
                else:
                    get_device_list_params[parameter_name] = param_value
            else:
                self.log("Parameter '{}' not found in config or is empty.".format(parameter), "DEBUG")

        self.log("Final get_device_list_params output: {}".format(get_device_list_params), "DEBUG")
        self.log("Completed get_device_list_params function.", "DEBUG")
        return get_device_list_params

    def get_device_ids_by_params(self, get_device_list_params):
        """Retrieves device IDs based on specified parameters from Cisco Catalyst Center.
        Parameters:
            get_device_list_params (dict): A dictionary of parameters to filter devices.
        Returns:
            dict: A dictionary mapping management IP addresses to instance IDs of reachable devices that are not Unified APs.
        Description:
            This method queries Cisco Catalyst Center to retrieve device information based on the provided filter parameters.
            It paginates through the results, filters out unreachable devices and Unified APs, and returns a dictionary of management IP addresses
            mapped to their instance IDs.
            Logs detailed information about the number of devices processed, skipped, and the final list of devices available for configuration backup.
        """
        mgmt_ip_to_instance_id_map = {}
        processed_device_count = 0
        skipped_device_count = 0
        # Define device families to skip
        skipped_device_families = {"Unified AP", "Wireless Sensor", "Third Party Device"}

        try:
            offset = 1
            limit = 500
            while True:
                # Update params with current offset and limit
                get_device_list_params.update({"offset": offset, "limit": limit})

                # Query Cisco Catalyst Center for device information using the parameters
                response = self.dnac._exec(
                    family="devices",
                    function="get_device_list",
                    op_modifies=True,
                    params=get_device_list_params,
                )
                self.log(
                    "Response received post 'get_device_list' API call with offset {0}: {1}".format(
                        offset, str(response)
                    ),
                    "DEBUG",
                )

                # Check if a valid response is received
                if not response.get("response"):
                    self.log(
                        "Exiting the loop because no devices were returned after increasing the offset. Current offset: {0}".format(
                            offset
                        )
                    )
                    break  # Exit loop if no devices are returned

                # Iterate over the devices in the response
                response = response.get("response")
                for device_info in response:
                    processed_device_count += 1
                    device_ip = device_info.get("managementIpAddress", "Unknown IP")

                    # Check if the device is reachable and managed
                    reachability = device_info.get("reachabilityStatus")
                    collection_status = device_info.get("collectionStatus")
                    device_family = device_info.get("family")

                    if reachability == "Reachable" and collection_status in ["Managed", "In Progress"]:
                        # Skip Unified AP devices
                        if device_family not in skipped_device_families:
                            device_id = device_info["id"]
                            mgmt_ip_to_instance_id_map[device_ip] = device_id
                        else:
                            skipped_device_count += 1
                            self.skipped_devices_list.append(device_ip)
                            self.log(
                                "Skipping device {0} as its family is: {1}.".format(device_ip, device_family),
                                "INFO"
                            )
                    else:
                        skipped_device_count += 1
                        self.skipped_devices_list.append(device_ip)
                        self.log(
                            "Skipping device {0} as its reachabilityStatus is '{1}' or collectionStatus is '{2}'.".format(
                                device_ip, reachability, collection_status
                            ),
                            "INFO"
                        )

                # Check if the response size is less than the limit
                if len(response) < limit:
                    self.log(
                        "Received less than limit ({0}) results, assuming last page. Exiting pagination.".format(
                            limit
                        ),
                        "DEBUG",
                    )
                    break

                # Increment offset for next batch
                offset += limit

            # Log the total number of devices processed and skipped
            self.log(
                "Total number of devices received: {0}".format(processed_device_count),
                "INFO",
            )
            self.log(
                "Number of devices that will be skipped: {0}".format(
                    skipped_device_count
                ),
                "INFO",
            )
            self.log(
                "Config Backup Operation can be performed on the following filtered devices: {0}".format(
                    len(mgmt_ip_to_instance_id_map)
                ),
                "INFO",
            )

        except Exception as e:
            # Log an error message if any exception occurs during the process
            self.log(
                "Error fetching device IDs from Cisco Catalyst Center. Error details: {0}".format(
                    str(e)
                ),
                "ERROR",
            )

        # Log an error if no reachable devices are found
        if not mgmt_ip_to_instance_id_map:
            self.log(
                "No reachable devices found among the provided parameters: {0}".format(
                    mgmt_ip_to_instance_id_map
                ),
                "ERROR",
            )

        return mgmt_ip_to_instance_id_map

    def get_device_id_list(self, config):
        """
        Retrieves device IDs based on site list or specified parameters from Cisco Catalyst Center.
        Parameters:
            config (dict): A dictionary containing device filter criteria, optionally including a list of sites.
        Returns:
            dict: A dictionary mapping management IP addresses to instance IDs of devices that match the provided criteria.
        Description:
            This method queries Cisco Catalyst Center to retrieve device information based on the provided filter criteria.
            If a list of sites is provided, it retrieves device IDs for those sites. Additionally, it can use other parameters to filter devices.
            Logs detailed information about the processing steps and the final list of device IDs retrieved.
        """
        # Initialize the mapping dictionary
        mgmt_ip_to_instance_id_map = {}

        # Check if site_list is provided in the config
        site_list = config.get("site_list")
        if site_list:
            self.log("List of site(s) provided in the input: {0}".format(site_list))

            # Use a set to ensure unique sites
            unique_sites = set(site_list)
            self.log(
                "Attempting to get Device Id(s) of all device(s) from the provided for site(s): {0}".format(
                    unique_sites
                ),
                "DEBUG",
            )

            # Retrieve device IDs for each site in the unique_sites set
            for site_name in unique_sites:
                site_mgmt_ip_to_instance_id_map, skipped_devices_list = (
                    self.get_reachable_devices_from_site(site_name)
                )
                self.skipped_devices_list.extend(skipped_devices_list)
                self.log(
                    "Retrieved following Device Id(s) of device(s): {0} from the provided site: {1}".format(
                        site_mgmt_ip_to_instance_id_map, site_name
                    ),
                    "DEBUG",
                )
                mgmt_ip_to_instance_id_map.update(site_mgmt_ip_to_instance_id_map)
                self.log(
                    "Devices from site: '{0}' that will be skipped: {1}".format(
                        site_name, skipped_devices_list
                    ),
                    "DEBUG",
                )

            # Get additional device list parameters excluding site_list
            get_device_list_params = self.get_device_list_params(config)
            if get_device_list_params:
                self.log(
                    "Attempting to get Device Id(s) of all device(s) using parameters(excluding site_list): {0}".format(
                        get_device_list_params
                    ),
                    "DEBUG",
                )
                params_mgmt_ip_to_instance_id_map = self.get_device_ids_by_params(
                    get_device_list_params
                )
                mgmt_ip_to_instance_id_map.update(params_mgmt_ip_to_instance_id_map)
                self.log(
                    "Retrieved following Device Id(s) of device(s): {0} from the provided parameters(excluding site_list).".format(
                        mgmt_ip_to_instance_id_map
                    ),
                    "DEBUG",
                )

        # If no site_list is provided, use other parameters to get device IDs
        else:
            get_device_list_params = self.get_device_list_params(config)
            self.log(
                "Attempting to get Device Id(s) of all device(s) using parameters(excluding site_list): {0}".format(
                    get_device_list_params
                ),
                "DEBUG",
            )
            params_mgmt_ip_to_instance_id_map = self.get_device_ids_by_params(
                get_device_list_params
            )
            mgmt_ip_to_instance_id_map.update(params_mgmt_ip_to_instance_id_map)
            self.log(
                "Retrieved following Device Id(s) of device(s): {0} from the provided parameters(excluding site_list).".format(
                    mgmt_ip_to_instance_id_map
                ),
                "DEBUG",
            )

        return mgmt_ip_to_instance_id_map

    def validate_ip4_address_list(self, ip_address_list):
        """
        Validates the list of IPv4 addresses provided in the playbook.
        Parameters:
            ip_address_list (list): A list of IPv4 addresses to be validated.
        Description:
            This method iterates through each IP address in the list and checks if it is a valid IPv4 address.
            If any address is found to be invalid, it logs an error message and fails the module.
            After validating all IP addresses, it logs a success message.
        """
        # Iterate through each IP address in the list to validate
        for ip in ip_address_list:
            # Check if the IP address is a valid IPv4 address
            if not self.is_valid_ipv4(ip):
                self.msg = "IP address: {0} is not valid".format(ip)
                self.log(self.msg, "ERROR")
                self.module.fail_json(self.msg)

        # Log a success message indicating all IP addresses are valid
        ip_address_list_str = ", ".join(ip_address_list)
        self.log(
            "Successfully validated the IP address(es): {0}".format(
                ip_address_list_str
            ),
            "DEBUG",
        )

    def validate_file_password(self, file_password):
        """
        Validates the provided file password against specified criteria.
        Parameters:
            file_password (str): The password to be validated.
        Description:
            This method checks if the provided password meets the criteria of having at least 8 characters,
            including at least one lowercase letter, one uppercase letter, one digit, and one special character.
            If the password does not meet these criteria, it logs a critical error message and fails the module.
        """
        # Define the regex pattern for a valid password
        password_pattern = r"^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[-=\\;,./~!@#$%^&*()_+{}[\]|:?\"]).{8,}$"

        # Log the user-defined password for debugging purposes
        self.log("User defined password is {0}".format(file_password), "DEBUG")

        # Check if the password matches the defined pattern
        if not re.match(password_pattern, file_password):
            msg = (
                "Invalid password input. The password must be at least 8 characters long and include "
                "at least one lowercase letter, one uppercase letter, one digit, and one special character "
                "from -=\\\\;,./~!@#$%^&*()_+{}[]|:?"
            )
            self.log(msg, "CRITICAL")
            self.module.fail_json(msg=msg)

    def password_generator(self):
        """
        Creates a password that matches Cisco Catalyst Center's requirements.
        Description:
            This method generates a password that meets the following criteria:
            - Minimum length of 8 characters.
            - Contains at least one lowercase letter.
            - Contains at least one uppercase letter.
            - Contains at least one digit.
            - Contains at least one special character from -=\\\\;,./~!@#$%^&*()_+{}[]|:?
            The generated password is shuffled to ensure randomness.
        """
        # Define the set of punctuation characters allowed in the password
        allowed_special_chars = "-=;,.~!@#$%^&*()_+{}[]|:?"

        # Combine allowed characters: punctuation, letters, and digits
        allowed_chars = allowed_special_chars + string.ascii_letters + string.digits

        # Create a list ensuring the password meets the criteria
        password_list = [
            random.choice(allowed_special_chars),
            random.choice(string.ascii_uppercase),
            random.choice(string.ascii_lowercase),
            random.choice(string.digits),
            random.choice(allowed_chars),
            random.choice(allowed_chars),
            random.choice(allowed_chars),
            random.choice(allowed_chars),
        ]

        # Form the password
        password = []
        random.shuffle(password_list)
        password = "".join(password_list)

        # Log the password generation event
        self.log("File password is generated using the password generator API", "INFO")

        return password

    def export_device_configurations_params(
        self, file_password, mgmt_ip_to_instance_id_map
    ):
        """
        Creates parameters for exporting device configurations from Cisco Catalyst Center.
        Parameters:
            file_password (str): The password to secure the exported device configurations.
            mgmt_ip_to_instance_id_map (dict): A dictionary mapping management IP addresses
            to instance IDs of devices.
        Returns:
            dict: A dictionary containing the device IDs and the file password for exporting device configurations.
        Description:
            This method constructs a dictionary of parameters required to export device configurations
            from Cisco Catalyst Center.
            The parameters include a list of device IDs and a password to secure the exported configurations.
        """
        # Construct the parameters dictionary for exporting device configurations
        export_device_configurations_params = {
            "deviceId": list(mgmt_ip_to_instance_id_map.values()),
            "password": file_password,
        }

        return export_device_configurations_params

    def export_device_configurations(self, export_device_configurations_params):
        """
        Exports device configurations from Cisco Catalyst Center using the provided parameters.
        Parameters:
            export_device_configurations_params (dict): A dictionary containing parameters for the export operation,
            including device IDs and a file password.
        Returns:
            str or None: The task ID of the export operation if successful, or None if the operation failed
            or no response was received.
        Description:
            This method initiates the export of device configurations from Cisco Catalyst Center using the provided parameters.
            It logs detailed information about the process, including the response from the API call and the
            task ID of the export operation.
            If an error occurs, it logs an error message, updates the result, and checks the return status.
        """
        task_id = self.get_taskid_post_api_call(
            "configuration_archive",
            "export_device_configurations",
            export_device_configurations_params,
        )
        return task_id

    def download_file(self, additional_status_url=None):
        """
        Downloads a file from Cisco Catalyst Center and stores it locally.
        Parameters:
            additionalStatusURL (str): The URL containing the file ID to be downloaded.
        Returns:
            tuple or None: A tuple containing the file ID and the file data if successful, or None if the download failed.
        Description:
            This method downloads a file from Cisco Catalyst Center using the provided URL, which contains the file ID.
            It logs the download process and checks the response from the API call. If successful, it returns the file
            ID and file data.
            If an error occurs, it logs an error message, updates the result, and checks the return status.
        """
        # Log the download URL for debugging purposes
        self.log(
            "Initiating download from URL: {0}".format(additional_status_url), "INFO"
        )
        file_id = additional_status_url.split("/")[-1]

        def try_download(function_name):
            try:
                response = self.dnac._exec(
                    family="file",
                    function=function_name,
                    op_modifies=True,
                    params={"file_id": file_id},
                )
                self.log(
                    "Response received post '{0}' API Call: {1}".format(
                        function_name, response
                    ),
                    "DEBUG",
                )

                # Check if response returned
                if response and response.data:
                    self.log(
                        "Download successful using function: {0}".format(function_name),
                        "DEBUG",
                    )
                    return (file_id, response.data)

                self.msg = "No response received post the '{0}' API call.".format(
                    function_name
                )
                return None

            except Exception as e:
                self.log(
                    "Exception occurred with '{0}': {1}".format(function_name, e),
                    "ERROR",
                )
                return None

        # Attempt the first function call
        result = try_download("download_a_file_by_fileid")
        if result is not None:
            self.log(
                "File download completed using 'download_a_file_by_fileid'", "INFO"
            )
            return result

        # Log and attempt the second function call if the first fails
        self.log(
            "Trying 'download_a_file_by_file_id' due to the exception in the previous call.",
            "INFO",
        )
        result = try_download("download_a_file_by_file_id")
        self.log("Type of result: {0}".format(type(result)), "DEBUG")
        if result is not None:
            self.log(
                "File download completed using 'download_a_file_by_file_id'", "INFO"
            )
            return result

        # Handle final failure case
        self.msg = (
            "The Backup Config file with File ID: {0} could not be downloaded.".format(
                file_id
            )
        )
        self.set_operation_result("failed", False, self.msg, "ERROR")
        self.check_return_status()

    def unzip_data(self, file_id, file_data):
        """
        Unzips the downloaded file data and stores it in the specified directory.
        Parameters:
            file_id (str): The ID of the file to be unzipped.
            file_data (bytes): The binary data of the downloaded file.
        Returns:
            bool: True if the file is successfully unzipped, otherwise it logs an error and fails the module.
        Description:
            This method takes the binary data of a downloaded file, unzips it using the provided file password, and stores the
            contents in the specified directory.
            It logs the unzipping process and handles any exceptions that may occur during the extraction.
        """
        # Create the directory path if it does not exist
        file_path = self.want.get("file_path")
        self.log("Creating directory path: {0}".format(file_path), "DEBUG")
        pathlib.Path(file_path).mkdir(parents=True, exist_ok=True)

        if not self.want.get("unzip_backup"):
            # Generate a timestamp and set the zipped file path
            timestamp = datetime.datetime.now().strftime("%d_%b_%Y_%H_%M_%S_%f")[:-3]
            zipped_file_path = "{0}/{1}_{2}.zip".format(file_path, timestamp, file_id)

            try:
                with open(zipped_file_path, "wb") as file:
                    file.write(file_data)

                self.log(
                    "Downloaded the zipped backup to {0} without unzipping.".format(
                        zipped_file_path
                    ),
                    "INFO",
                )
                return True
            except OSError as e:
                self.log(
                    "Failed to write zipped backup to {0}. Error: {1}".format(
                        zipped_file_path, str(e)
                    ),
                    "ERROR",
                )
                return False

        try:
            # Convert the binary file data to a BytesIO object for processing
            zip_data = BytesIO(file_data)
            self.log("Collected ZIP Data for file with ID: {0}".format(file_id), "INFO")

            # Unzip the file using the provided file password
            self.log(
                "Unzipping Backup Config file with file ID: {0} after completion of download.".format(
                    file_id
                ),
                "INFO",
            )
            file_password = self.want.get("file_password")
            with pyzipper.AESZipFile(zip_data, "r") as f:
                f.pwd = bytes(file_password, encoding="utf-8")
                f.extractall(path=str(file_path))
            return True
        except Exception as e:
            self.msg = "Error in unzipping Backup Config file with file ID: {0}. Error: {1}".format(
                file_id, e
            )
            self.set_operation_result("failed", False, self.msg, "ERROR")
            self.check_return_status()

    def get_export_device_config_task_status(self, task_id):
        """
        Checks the status of a device configuration export task in Cisco Catalyst Center.
        Parameters:
            task_id (str): The ID of the task to check the status for.
        Returns:
            self: The instance of the class, potentially updated with task results.
        Description:
            This method repeatedly checks the status of a device configuration export task using the provided task ID.
            It logs progress, handles errors, and performs additional tasks such as downloading and unzipping the
            file if the task completes successfully.
        """
        task_name = "Backup Device Configuration"
        success_msg = "{0} Task with task ID {1} completed successfully. Exiting the loop.".format(
            task_name, task_id
        )
        if self.dnac_version <= self.version_2_3_5_3:
            progress_validation = (
                "Device configuration Successfully exported as password protected ZIP"
            )
            failure_msg = "An error occurred while performing {0} task with task ID {1} for export_device_configurations_params: {2}".format(
                task_name, task_id, self.want.get("export_device_configurations_params")
            )
            self.get_task_status_from_task_by_id(
                task_id,
                task_name,
                failure_msg,
                success_msg,
                progress_validation=progress_validation,
            )
        else:
            self.get_task_status_from_tasks_by_id(task_id, task_name, success_msg)

        if self.status == "success":
            self.log(
                "Task '{0}' completed successfully for task ID {1}.".format(
                    task_name, task_id
                ),
                "INFO",
            )
            if self.dnac_version <= self.version_2_3_5_3:
                response = self.get_task_details(task_id)
                additional_status_url = response.get("additionalStatusURL")
            else:
                response = self.get_tasks_by_id(task_id)
                additional_status_url = response.get("resultLocation")

            if not additional_status_url:
                self.msg = "Error retrieving the Device Config Backup file ID for task ID {0}".format(
                    task_id
                )
                self.fail_and_exit(self.msg)
            self.log(
                "Additional status URL retrieved: {0}".format(additional_status_url),
                "DEBUG",
            )

            # Perform additional tasks after breaking the loop
            mgmt_ip_to_instance_id_map = self.want.get("mgmt_ip_to_instance_id_map")

            # Download the file using the additional status URL
            self.log(
                "Downloading the Device Config Backup file from {0}.".format(
                    additional_status_url
                ),
                "DEBUG",
            )
            file_id, downloaded_file = self.download_file(
                additional_status_url=additional_status_url
            )
            self.log("Retrived file data for file ID: {0}.".format(file_id), "DEBUG")
            if not downloaded_file:
                self.msg = "Error downloading Device Config Backup file(s) with file ID: {0}. ".format(
                    file_id
                )
                self.fail_and_exit(self.msg)

            # Unzip the downloaded file
            self.log(
                "Unzipping the downloaded Device Config Backup file(s) for file ID: {0}.".format(
                    file_id
                ),
                "DEBUG",
            )
            download_status = self.unzip_data(file_id, downloaded_file)
            if download_status:
                self.log(
                    "{0} task has been successfully performed on {1} device(s): {2}.".format(
                        task_name,
                        len(mgmt_ip_to_instance_id_map),
                        list(mgmt_ip_to_instance_id_map.keys()),
                    ),
                    "INFO",
                )
                self.log(
                    "{0} task has been skipped for {1} device(s): {2}".format(
                        task_name,
                        len(self.skipped_devices_list),
                        self.skipped_devices_list,
                    ),
                    "INFO",
                )
                self.msg = (
                    "{0} task has been successfully performed on {1} device(s) and skipped on {2} device(s). "
                    "The backup configuration files can be found at: {3}.".format(
                        task_name,
                        len(mgmt_ip_to_instance_id_map),
                        len(self.skipped_devices_list),
                        pathlib.Path(self.want.get("file_path")).resolve(),
                    )
                )

                # Append password information if unzipping is not required
                if not self.want.get("unzip_backup", False):
                    self.msg += " The password to unzip the files is: '{0}'.".format(
                        self.want.get("file_password")
                    )
                self.set_operation_result("success", True, self.msg, "INFO")
            else:
                self.msg = "Error unzipping Device Config Backup file(s) with file ID: {0}. ".format(
                    file_id
                )
                self.fail_and_exit(self.msg)

        return self

    def get_network_device_configuration_file_details(self, mgmt_ip_to_instance_id_map):
        """
        Retrieves the latest configuration file details for a list of network devices.

        If no 'file_types' are specified in the playbook input (self.want), the function defaults to
        retrieving the following configuration file types: VLAN, STARTUPCONFIG, and RUNNINGCONFIG.

        Parameters:
            mgmt_ip_to_instance_id_map (dict): A dictionary mapping management IP addresses (str)
                                            to corresponding network device instance IDs (str).

        Returns:
            list or None:
                - A list of dictionaries for each device, each containing:
                    - device_id (str): The device instance ID.
                    - ip_address (str): The management IP address.
                    - file_ids (list): List of retrieved file IDs.
                    - file_types (list): List of successfully collected config file types.
                - Returns None if no configuration files are found for any device.

        Raises:
            Terminates the module execution with an error message if:
                - Invalid file types are specified.
                - An unexpected exception occurs during processing.
        """
        self.log("Retrieving latest configuration file details for network devices.", "INFO")

        valid_file_types = ["VLAN", "STARTUPCONFIG", "RUNNINGCONFIG"]
        input_file_types = self.want.get("file_types")

        if not input_file_types or not isinstance(input_file_types, list):
            file_types = valid_file_types
            self.log("No file_types specified in the playbook. Defaulting to {0}".format(file_types), "INFO")
        else:
            file_types = []
            for ftype in input_file_types:
                upper_ftype = ftype.upper()
                if upper_ftype not in valid_file_types:
                    msg = "Invalid file_type: {0} given in the input. Valid file_types: {1}".format(
                        ftype, valid_file_types
                    )
                    self.fail_and_exit(msg)
                file_types.append(upper_ftype)

        self.log("Validated file_types received: {0}".format(file_types), "DEBUG")

        try:
            self.log("Processing device list to retrieve configuration file details.", "DEBUG")

            filter_file_ids = []

            for ip_address in self.want.get("mgmt_ip_to_instance_id_map"):
                device_id = mgmt_ip_to_instance_id_map.get(ip_address)
                self.log("Processing device IP: {0}, Device ID: {1}".format(ip_address, device_id), "DEBUG")

                file_ids = []
                collected_types = []

                for file_type in file_types:
                    self.log("Fetching latest '{0}' config for Device ID: {1}".format(file_type, device_id), "DEBUG")

                    response = self.execute_get_request(
                        "configuration_archive",
                        "get_network_device_configuration_file_details",
                        {
                            "networkDeviceId": device_id,
                            "fileType": file_type,
                            "offset": 1,
                            "limit": 1
                        }
                    )

                    if response and response.get("response"):
                        file_id = response["response"][0].get("id")
                        if file_id:
                            file_ids.append(file_id)
                            collected_types.append(file_type)
                            self.log("Retrieved File ID: {0} for Device ID: {1}, Type: {2}".format(
                                file_id, device_id, file_type), "DEBUG")
                        else:
                            self.log("No File ID in response for Device ID: {0}, Type: {1}".format(
                                device_id, file_type), "WARNING")
                    else:
                        self.log("No config files found for Device ID: {0}, Type: {1}".format(
                            device_id, file_type), "DEBUG")

                filter_file_ids.append({
                    "device_id": device_id,
                    "ip_address": ip_address,
                    "file_ids": file_ids,
                    "file_types": collected_types
                })

            if filter_file_ids:
                self.log("Final configuration file details retrieved:\n{0}".format(
                    self.pprint(filter_file_ids)), "INFO")
                return filter_file_ids

            self.log("No configuration files found for any device.", "WARNING")
            return None

        except Exception as e:
            msg = "An error occurred in get_network_device_configuration_file_details: {0}".format(str(e))
            self.log(msg, "ERROR")
            self.fail_and_exit(msg)

    def get_configuration_file_ids(self, mgmt_ip_to_instance_id_map, file_password):
        """
        Retrieves only the 'id' values from the configuration file details response.

        Parameters:
            mgmt_ip_to_instance_id_map (dict): Mapping of management IPs to device instance IDs.
            file_password (str): Password required for unmasking configuration files.

        Returns:
            list: List of 'id' strings extracted from the configuration file details response.
        """
        self.log("Retrieving configuration file IDs from the configuration file details.", "INFO")

        response = self.get_network_device_configuration_file_details(mgmt_ip_to_instance_id_map)

        if not response or not isinstance(response, list):
            self.log("No configuration file details found or invalid format returned.", "WARNING")
            return []

        id_list = []

        for item in response:
            if not isinstance(item, dict):
                self.log("Skipping invalid item. Expected a dictionary but got {}...".format(type(item).__name__), "WARNING")
                continue

            file_ids = item.get("file_ids", [])
            file_types = item.get("file_types", [])
            ip_address = item.get("ip_address")

            if file_ids and isinstance(file_ids, list):
                self.log(f"Processing file IDs: {file_ids} with types: {file_types} for device {ip_address}.", "DEBUG")
                id_list.extend(file_ids)

                self.log("Processing file IDs: {0} with types: {1}".format(file_ids, file_types), "DEBUG")
                unmasked_params = self.download_unmasked_raw_device_configuration(file_ids, file_types, file_password)

                self.log(f"Unmasked parameters retrieved for device {ip_address}: {unmasked_params}", "INFO")
                self.log("Added file IDs: {0}".format(file_ids), "DEBUG")
            else:
                self.log("No valid file_ids list found in item: {0}".format(item), "DEBUG")

        if id_list:
            self.log("Extracted file IDs successfully: {0}".format(id_list), "INFO")
        else:
            self.log("No valid file IDs extracted from configuration file details.", "WARNING")

        return id_list

    def download_unmasked_raw_device_configuration(self, id_list, file_password):
        """
        Downloads the unmasked (raw) configuration ZIP file for the provided file IDs.

        Parameters:
            id_list (list): List of configuration file IDs.
            file_password (str): Password to decrypt the downloaded ZIP files.

        Returns:
            bytes: Binary ZIP content of the first successfully downloaded configuration file.
                Returns None if no valid data is received.
        """
        self.log("Starting download_unmasked_raw_device_configuration", "INFO")

        if not id_list or not file_password:
            msg = (
                "Missing required parameters: "
                f"'id_list' is {'not provided' if not id_list else 'provided'}, "
                f"'file_password' is {'not set' if not file_password else 'set'}"
            )
            self.fail_and_exit(msg)

        try:
            for file_id in id_list:
                payload = {"id": file_id, "password": file_password}
                self.log(f"Requesting export for file ID: {file_id}", "INFO")

                response = self.dnac._exec(
                    family="configuration_archive",
                    function="download_unmaskedraw_device_configuration_as_zip",
                    op_modifies=True,
                    params=payload
                )

                if response and getattr(response, "data", None):
                    self.log(
                        f"Received data for file ID {file_id}: type={response.data.__class__.__name__}, size={len(response.data)}",
                        "DEBUG"
                    )
                    return response.data
                else:
                    self.log(f"No valid data received for file ID: {file_id}", "WARNING")

            self.log("No valid unmasked configuration file received for any file ID.", "WARNING")
            return None

        except Exception as e:
            error_msg = f"Error in download_unmasked_raw_device_configuration: {e}"
            self.log(error_msg, "ERROR")
            self.set_operation_result("failed", False, error_msg, "ERROR").check_return_status()
            return None

    def download_masked_device_configuration(self, id_list):
        """
        Downloads the masked (secured) configuration ZIP file for each provided file ID using the Catalyst Center API.

        Parameters:
            id_list (list): A list of configuration file IDs to download.

        Returns:
            bytes: A ZIP archive in binary format containing the first successfully downloaded configuration file.
                Returns None if no valid response is received.
        """
        self.log("Starting download_masked_device_configuration", "INFO")

        if not id_list:
            msg = f"Missing required 'id_list' parameter: id_list={id_list}"
            self.fail_and_exit(msg)

        try:
            for file_id in id_list:
                payload = {"id": file_id}
                self.log(f"Requesting download for file ID: {file_id} with payload: {payload}", "INFO")

                response = self.dnac._exec(
                    family="configuration_archive",
                    function="download_masked_device_configuration",
                    op_modifies=True,
                    params=payload
                )

                if response and hasattr(response, "data") and response.data:
                    self.log(f"Received data for file ID {file_id}: type={response.data.__class__.__name__}, length={len(response.data)}", "DEBUG")
                    return response.data
                else:
                    self.log(f"No valid data received for file ID: {file_id}", "WARNING")

            self.log("No valid masked configuration file received for any file ID.", "WARNING")
            return None

        except Exception as e:
            error_msg = f"Error in download_masked_device_configuration: {e}"
            self.log(error_msg, "ERROR")
            self.set_operation_result("failed", False, error_msg, "ERROR").check_return_status()
            return None

    def download_unmasked_and_masked_configuration(self):
        """
        Downloads configuration files (masked or unmasked) for devices and saves them locally.

        Parameters:
            None (uses values from self.want):
                - file_password: Password for ZIP files (if needed).
                - file_path: Folder to save files (default: 'backup').
                - file_types: List of config types (e.g., STARTUPCONFIG).
                - unzip_backup: If True, downloads masked config as text.
                - mgmt_ip_to_instance_id_map: Device IP to ID mapping.

        Returns:
            None

        Description:
            - Gets latest config file IDs for each device.
            - Creates folders to store the files.
            - Downloads either unmasked (ZIP) or masked (text) config files.
            - Saves the files in the specified directory.
            - Logs status and sets final result message.
        """
        params = self.want
        self.log("Configuration Params: {}".format(params), "DEBUG")

        file_password = params.get("file_password")
        base_backup_path = params.get("file_path", "backup")
        file_types = params.get("file_types")
        unzip_required = params.get("unzip_backup", False)

        os.makedirs(base_backup_path, exist_ok=True)

        mgmt_ip_to_instance_id_map = params.get("mgmt_ip_to_instance_id_map", {})
        self.log("Management IP to Instance ID Map: {}".format(mgmt_ip_to_instance_id_map), "DEBUG")

        self.log("Retrieving configuration file details for the devices...", "INFO")
        try:
            file_details_list = self.get_network_device_configuration_file_details(mgmt_ip_to_instance_id_map)
            self.log(f"Configuration file details retrieved: {file_details_list}", "DEBUG")
        except Exception as e:
            self.log(f"Error retrieving configuration file details: {e}", "ERROR")
            return

        if not file_details_list:
            self.log("No configuration file details found. Aborting the operation.", "ERROR")
            return

        for device_files in file_details_list:
            ip_address = device_files.get("ip_address")
            file_ids = device_files.get("file_ids")

            if not file_ids:
                self.log("No file IDs found for IP {}, skipping this device.".format(ip_address), "WARNING")
                continue

            date_str = datetime.datetime.now().strftime("%d_%b_%Y")
            ip_folder_name = "{}_{}".format(date_str, ip_address.replace('.', '_'))
            target_dir = os.path.join(base_backup_path, ip_folder_name)
            os.makedirs(target_dir, exist_ok=True)

            for file_id in file_ids:
                self.log(f"Downloading configuration data for file ID {file_id} (Device IP: {ip_address})...", "INFO")
                if not isinstance(file_id, str) or len(file_id) < 36:
                    self.log(f"Invalid file ID: {file_id}, skipping.", "WARNING")
                    continue

                if unzip_required:
                    config_data = self.download_masked_device_configuration(id_list=[file_id])
                    if not config_data:
                        self.log("No configuration data for file ID {} (masked), skipping.".format(file_id), "WARNING")
                        continue

                    output_file = os.path.join(target_dir, f"{file_id}.txt")
                    try:
                        with open(output_file, "wb") as f:
                            f.write(config_data)
                        self.log("Masked configuration saved: {}".format(output_file), "INFO")
                    except Exception as e:
                        self.log(f"Error writing masked config for {file_id}: {e}", "ERROR")

                else:
                    config_data = self.download_unmasked_raw_device_configuration(
                        id_list=[file_id],
                        file_password=file_password
                    )
                    if not config_data:
                        self.log("No configuration data for file ID {} (unmasked), skipping.".format(file_id), "WARNING")
                        continue

                    original_file_path = params.get("file_path")
                    try:
                        self.want["file_path"] = target_dir
                        success = self.unzip_data(file_id, config_data)
                    finally:
                        self.want["file_path"] = original_file_path

                    if not success:
                        self.log("Failed to process ZIP for file ID {} at IP {}".format(file_id, ip_address), "ERROR")

        total_devices = len(mgmt_ip_to_instance_id_map)
        processed_devices = len(file_details_list)
        skipped_devices = total_devices - processed_devices
        abs_backup_path = os.path.abspath(base_backup_path)

        log_msg = (
            f"Configuration backup operation completed: {processed_devices} device(s) processed successfully, "
            f"{skipped_devices} device(s) skipped. Backup files saved at: {abs_backup_path}."
        )

        if not unzip_required and file_password:
            log_msg += f" Password to unzip files: '{file_password}'."

        self.log("Completed the process of downloading and saving unmasked and masked configuration files", "INFO")
        self.set_operation_result("success", True, log_msg, "INFO")

    def get_want(self, config):
        """
        Prepares the desired state (want) based on the provided configuration.
        Parameters:
            config (dict): A dictionary containing the configuration parameters.
        Returns:
            self: The instance of the class, potentially updated with the desired state.
        Description:
            This method processes the provided configuration to prepare the desired state (want).
            It validates the IP address list and file password, generates a new password if none is provided,
            retrieves device IDs, and updates the desired state with necessary parameters.
        """
        self.log("Starting the process to prepare the desired state (want) based on the provided configuration.", "INFO")

        self.want = {}

        # Retrieve and log configuration parameters
        file_path = config.get("file_path", "backup")  # Default to 'backup' if not provided
        file_password = config.get("file_password")
        ip_address_list = config.get("ip_address_list")
        file_types = config.get("config_file_types")
        if file_types:
            current_version = self.get_ccc_version()
            self.log(f"Current Catalyst Center Version: {current_version}", "DEBUG")
            if self.compare_dnac_versions(current_version, "2.3.7.9") < 0:
                self.fail_and_exit(
                    "The 'config_file_types' parameter is not supported for Catalyst Center version 2.3.7.6. "
                    "It is supported from version 2.3.7.9 onwards."
                )

            self.log("Converting file types to uppercase for consistency.", "DEBUG")
            upper_file_types = []
            for ftype in file_types:
                upper_file_types.append(ftype.upper())
            file_types = upper_file_types

            # Validate conflict if 'ALL' is selected with other types
            if "ALL" in file_types and len(file_types) > 1:
                self.fail_and_exit(
                    "Invalid 'config_file_types' selection: Please select either 'All' or specific file types "
                    "(VLAN, STARTUPCONFIG, RUNNINGCONFIG), not both."
                )

            # If 'ALL' is selected, expand to all supported file types
            if "ALL" in file_types:
                self.log("Expanding 'ALL' to include all supported file types: VLAN, STARTUPCONFIG, RUNNINGCONFIG.", "DEBUG")
                file_types = ["VLAN", "STARTUPCONFIG", "RUNNINGCONFIG"]

        # Validate the IP address list if provided
        if ip_address_list:
            self.log(f"Validating the IP address list: {ip_address_list}", "INFO")
            self.validate_ip4_address_list(ip_address_list)
            self.log("IP address list validation completed successfully.", "INFO")

        if file_password:
            self.log("Validating the provided file password.", "INFO")
            self.validate_file_password(file_password)
            self.log("File password validation completed successfully.", "INFO")
        else:
            file_password = self.password_generator()
            self.log("No file password provided. Generated a new password.", "INFO")

        self.log("Retrieving the device ID list based on the provided IP addresses.", "INFO")
        mgmt_ip_to_instance_id_map = self.get_device_id_list(config)

        if not mgmt_ip_to_instance_id_map:
            self.msg = f"No reachable devices found among the provided parameters: {config}"
            self.set_operation_result("ok", False, self.msg, "INFO")
            self.log(f"Process completed: {self.msg}", "INFO")
            return self

        self.log(
            f"Retrieved {len(mgmt_ip_to_instance_id_map)} device(s) with their corresponding instance IDs: "
            f"{mgmt_ip_to_instance_id_map}", "INFO"
        )

        self.log("Preparing the desired state (want).", "INFO")
        self.want["export_device_configurations_params"] = self.export_device_configurations_params(
            file_password, mgmt_ip_to_instance_id_map
        )
        self.want["mgmt_ip_to_instance_id_map"] = mgmt_ip_to_instance_id_map
        self.want["file_password"] = file_password
        self.want["file_path"] = file_path
        self.want["unzip_backup"] = config.get("unzip_backup")
        self.want["file_types"] = file_types

        self.log(f"Desired State (want) has been prepared: {str(self.want)}", "INFO")
        self.log("Completed the process to prepare the desired state (want).", "INFO")

        return self

    def get_diff_merged(self):
        """
        This method is designed to Execute and Monitor Device Configuration Export Actions.
        Returns:
            self (object): An instance of the class used for continuing the chaining of method calls.
        Description:
            This method logs its execution, iterates through a predefined map of actions and their corresponding status-checking functions,
            executes the required actions based on provided parameters, and verifies the success of each action by checking its status.
        """
        self.log("Executing the get_diff_merged function", "DEBUG")

        if not self.want:
            self.log("No configuration found to backup. Skipping the operation.", "DEBUG")
            return self

        if self.compare_dnac_versions(self.get_ccc_version(), "2.3.7.6") <= 0:
            action_map = {
                "export_device_configurations_params": (
                    self.export_device_configurations,
                    self.get_export_device_config_task_status,
                ),
            }

            for action_param, (action_func, status_func) in action_map.items():
                if self.want.get(action_param):
                    result_task_id = action_func(self.want.get(action_param))
                    status_func(result_task_id).check_return_status()
        else:
            self.log("Detected DNAC version newer than 2.3.7.6 — running download_unmasked_and_masked_configuration()", "INFO")
            self.download_unmasked_and_masked_configuration()

        return self

    def verify_diff_merged(self):
        """
        This method is designed to Verify the Success of Device Configuration Backup Operation.
        Returns:
            self (object): An instance of the class used for continuing the chaining of method calls.
        Description:
            This method validates the Cisco Catalyst Center configuration for a merged state by checking if any backup files were created
            within a specified time window. It logs the desired state, checks the modification time of files in the specified directory,
            and sets the operation status based on whether recent backup files are found.
        """

        if not self.want:
            self.log("No configuration found to verify. Skipping the operation.", "DEBUG")
            return self

        file_path = self.want.get("file_path")
        self.log("File Path: {0}".format(file_path))
        if file_path:
            abs_file_path = pathlib.Path(file_path).resolve()
        self.log("Desired State (want): {0}".format(str(self.want)), "INFO")

        # Define the time window in seconds to check for recently modified files
        window_seconds = 10
        current_time = time.time()
        window_start_time = current_time - window_seconds

        # List of files modified within the specified time window
        files_modified_within_window = []

        try:
            for f in abs_file_path.iterdir():
                if f.stat().st_mtime > window_start_time:
                    files_modified_within_window.append(f.name)
        except Exception as e:
            self.msg = (
                "An error occurred while verifying the success of the backup configuration operation. "
                "The Device Config Backup operation may not have been successful since the backup files "
                "were not found at the specified path. Error: {0}".format(str(e))
            )
            self.set_operation_result("failed", False, self.msg, "ERROR")
            return self

        # Check if there are any files modified within the window
        if len(files_modified_within_window) > 0:
            self.log(
                "Verified the success of the Device Config Backup operation. Back up has been taken in the following files {0}".format(
                    str(files_modified_within_window)
                ),
                "INFO",
            )
            self.status = "success"
        else:
            self.log(
                "The Device Config Backup operation may not have been successful since back up files not found at path: {0}".format(
                    abs_file_path
                ),
                "WARNING",
            )

        return self


def main():
    """
    main entry point for module execution
    """
    # Define the specification for the module"s arguments
    element_spec = {
        "dnac_host": {"required": True, "type": "str"},
        "dnac_port": {"type": "str", "default": "443"},
        "dnac_username": {"type": "str", "default": "admin", "aliases": ["user"]},
        "dnac_password": {"type": "str", "no_log": True},
        "dnac_verify": {"type": "bool", "default": "True"},
        "dnac_version": {"type": "str", "default": "2.2.3.3"},
        "dnac_debug": {"type": "bool", "default": False},
        "dnac_log": {"type": "bool", "default": False},
        "dnac_log_level": {"type": "str", "default": "WARNING"},
        "dnac_log_file_path": {"type": "str", "default": "dnac.log"},
        "dnac_log_append": {"type": "bool", "default": True},
        "config_verify": {"type": "bool", "default": False},
        "dnac_api_task_timeout": {"type": "int", "default": 1200},
        "dnac_task_poll_interval": {"type": "int", "default": 2},
        "validate_response_schema": {"type": "bool", "default": True},
        "config": {"required": True, "type": "list", "elements": "dict"},
        "state": {"default": "merged", "choices": ["merged"]},
    }

    # Initialize the Ansible module with the provided argument specifications
    module = AnsibleModule(argument_spec=element_spec, supports_check_mode=False)

    # Initialize the NetworkCompliance object with the module
    ccc_device_configs_backup = DeviceConfigsBackup(module)

    if (
        ccc_device_configs_backup.compare_dnac_versions(
            ccc_device_configs_backup.get_ccc_version(), "2.3.7.6"
        )
        < 0
    ):
        ccc_device_configs_backup.msg = (
            "The specified version '{0}' does not support the 'Device Configuration Backup' feature. Supported versions start "
            "  from '2.3.7.6' onwards. Version '2.3.7.6' introduces APIs for taking configuration backups of reachable"
            " devices.".format(ccc_device_configs_backup.get_ccc_version())
        )
        ccc_device_configs_backup.set_operation_result(
            "failed", False, ccc_device_configs_backup.msg, "ERROR"
        ).check_return_status()

    # Get the state parameter from the provided parameters
    state = ccc_device_configs_backup.params.get("state")

    # Check if the state is valid
    if state not in ccc_device_configs_backup.supported_states:
        ccc_device_configs_backup.status = "invalid"
        ccc_device_configs_backup.msg = "State {0} is invalid".format(state)
        ccc_device_configs_backup.check_return_status()

    # Get the config_verify parameter from the provided parameters
    config_verify = ccc_device_configs_backup.params.get("config_verify")

    # Validate the input parameters and check the return status
    ccc_device_configs_backup.validate_input().check_return_status()

    # Iterate over the validated configuration parameters
    for config in ccc_device_configs_backup.validated_config:
        ccc_device_configs_backup.reset_values()
        ccc_device_configs_backup.get_want(config).check_return_status()
        ccc_device_configs_backup.get_diff_state_apply[state]().check_return_status()
        if config_verify:
            ccc_device_configs_backup.verify_diff_state_apply[
                state
            ]().check_return_status()

    # Exit with the result obtained from the NetworkCompliance object
    module.exit_json(**ccc_device_configs_backup.result)


if __name__ == "__main__":
    main()
